--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, smack = ...
local zu = smack.zu

local MacroFrame = {}
smack.MacroFrame = MacroFrame

local frameUID = 0

local MacroEnvSetup =
[[
    v = table.new()
]]

local function wrapScript( timeslicer, self )
    local snippet = smack.GetCompiledMacro( self.macro.index, timeslicer )

    if snippet then
        self.frame:WrapScript( self.frame, "OnClick", snippet )
    else
        self.frame:UnwrapScript( self.frame, "OnClick" )
        self.frame:SetAttribute( "macrotext", "" )
    end
end

function MacroFrame:update( timeslicer )
    if not self.macro.properties.enabled then
        self:release()
        return
    end

    smack.WaitOutOfCombat()

    -- We are not necessarily called by a coroutine, and if we are not
    -- the wait above won't work.
    if InCombatLockdown() then
        smack.QueueMacroForUpdate( self )
        return
    end

    if not self.frame then
        self.frame = CreateFrame( "Button", "SmackMacro_" .. self.uid, smack.MainMacroFrame,
            "SecureHandlerBaseTemplate,SecureActionButtonTemplate" )
        self.frame:SetAttribute( "type", "macro" )
        self.frame:RegisterForClicks( "AnyUp", "AnyDown" )
    end

    self.frame:Execute( MacroEnvSetup )
    self.frame:SetAttribute( "state", 0 )
    self.frame:UnwrapScript( self.frame, "OnClick" )

    if timeslicer then
        wrapScript( timeslicer, self )
    else
        zu.TimeSlice( wrapScript, 1, self )
    end

    local underscoreName = self.name:gsub( " ", "_" )

    smack.WaitOutOfCombat()

    if self.name then
        self.mainMacroFrame:SetAttribute( "*macrotext-" .. underscoreName )
    end

    self.name = self.macro.name
    self.mainMacroFrame:SetAttribute( "*macrotext-" .. underscoreName, "/click " .. "SmackMacro_" .. self.uid .. " LeftButton 1" )
    SetCVar( "ActionButtonUseKeyDown", 1 )

    self.frame:Show()
end

function MacroFrame:new( macro, timeslicer )
    local o = {}

    o.name = macro.name
    o.mainMacroFrame = smack.MainMacroFrame
    o.uid = frameUID
    setmetatable( o, { __index = self } )

    frameUID = frameUID + 1

    o:reuse( macro, timeslicer )
    return o
end

function MacroFrame:reuse( macro, timeslicer )
    self.macro = macro
    smack.AddListener( macro.updateEvent, self, MacroFrame.update )
    self:update( timeslicer )
end

function MacroFrame:release()
    if self.frame then
        self.frame:Hide()
        self.frame:UnwrapScript( self.frame, "OnClick" )
    end
    smack.RemoveListener( self.macro.updateEvent, self )
end
