--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, smack = ...

local classTags = {}
local specTags = {}

-- Setup class tags
local _, playerClass = UnitClassBase( "player" )
local playerCInfo = C_CreatureInfo.GetClassInfo( playerClass )

for i=1, GetNumClasses(), 1 do
    local cinfo = C_CreatureInfo.GetClassInfo( i )
    if( cinfo ~= nil ) then
        local cname = cinfo.classFile:lower()
        local _, _, _, color = GetClassColor( cinfo.classFile )

        classTags[cname] =
        {
            name = cname,
            displayName = '|c' .. color .. cname .. '|r',
            colorTag = '|c' .. color,
            playerClass == i
        }
    end
end

-- Setup specialization tags
local specs = smack.SpecsDB

for i = 1, #specs, 1 do
    local s = specs[i]
    local name = s[1]:lower()
    local classes = s[2]

    local ti =
    {
        name = name,
        specId = classes[playerClass]
    }

    -- We can't even easily get the number of non array entries in a table
    -- shit-ass language
    local numClasses = 0
    for _,_ in pairs( classes ) do
        numClasses = numClasses + 1
        if numClasses > 1 then
            break
        end
    end

    if numClasses == 1 then
        local classIndex = next( classes )
        local cinfo = C_CreatureInfo.GetClassInfo( classIndex )
        local cname = cinfo.classFile:lower()
        local _, _, _, color = GetClassColor( cinfo.classFile )
        ti.displayName = '|c' .. color .. name .. '|r'
        ti.colorTag = '|c' .. color
        ti.cname = cname
    else
        ti.displayName = name
    end

    specTags[name] = ti
end

local function GetSpecialTagInfos( tag )
    tag = tag:lower()
    return classTags[tag] or specTags[tag]
end

smack.GetSpecialTagInfos = GetSpecialTagInfos

-- Check the tags of a macro. Return true if the macro
-- should be enabled.
function smack.SpecialTagsCheckMacroEnable( tags )
    local classFound = false
    local classOk = true

    local specFound = false
    local specOk = true

    local specId = GetSpecializationInfo( GetSpecialization() )

    for i=1,#tags,1 do
        local tag = tags[i]:lower()
        local taginfo = classTags[tag]

        if taginfo then
            if taginfo[1] then
                classFound = true
            end
            classOk = classFound
        else
            taginfo = specTags[tag]
            if taginfo then
                if taginfo.specId and taginfo.specId == specId then
                    specFound = true
                end
                specOk = specFound
            end
        end
    end

    return classOk and specOk
end

-- Check if the macro is enabled only for one class,
-- returns that class name and color tag if so.
function smack.SpecialTagsCheckForSpecificClass( tags )
    local cname, colorTag

    for i=1,#tags,1 do
        local tag = tags[i]:lower()
        local taginfo = classTags[tag]
        if taginfo then
            if cname and cname ~= taginfo.name then
                return
            end
            cname = taginfo.name
            colorTag = taginfo.colorTag
        end

        taginfo = specTags[tag]
        if taginfo and taginfo.cname then
            if cname and cname ~= taginfo.cname then
                return
            end

            cname = cname or taginfo.cname
            colorTag = colorTag or taginfo.colorTag
        end
    end

    return cname and { cname = cname, colorTag = colorTag } or nil
end

-- Check if the macro is for the current class, spec notwithstanding
function smack.SpecialTagsCheckIsForCurrentClass( tags )
    local classFound = false
    local classOk = true

    local specFound = false
    local specOk = true

    local cname = playerCInfo.classFile:lower()

    for i=1,#tags,1 do
        local tag = tags[i]:lower()
        local taginfo = classTags[tag]

        if taginfo then
            if taginfo[1] then
                classFound = true
            end
            classOk = classFound
        else
            taginfo = specTags[tag]
            if taginfo and taginfo.cname then
                if taginfo.cname == cname then
                    specFound = true
                end
                specOk = specFound
            end
        end
    end

    return classOk and specOk
end
