--
-- Zlodo's utilities
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local yield = coroutine.yield
local resume = coroutine.resume
local running = coroutine.running
local wrap = coroutine.wrap

-- Launch the given function, time slicing its work
-- so that it never runs more than roughly allowedTime
-- (in ms) per frame.
--
-- The called function receives a timeslicing function as its first
-- parameter and is expected to call it regularly.
--
-- This returns a handle that can be passed to CancelTimeSlicedFunction
function zu.TimeSlice( func, allowedTime, ... )
    local handle = {}
    local deadline = debugprofilestop() + allowedTime

    local wrapped = wrap( func )

    local function tsYield()
        if debugprofilestop() >= deadline then
            local coro = running()
            local function resumeWork()
                -- Don't resume if we were canceled
                if not handle.canceled then
                    deadline = debugprofilestop() + allowedTime
                    resume( coro )
                end
            end

            C_Timer.After( 0, resumeWork )
            yield()
        end
    end

    wrapped( tsYield, ... )
    return handle
end

function zu.CancelTimeSlicedFunction( handle )
    handle.canceled = true
end
