--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, compiler = ...

function compiler.FoldConstants( expr )
    if expr[1] ~= "bool" or expr[3] == compiler.CFG.ExprOpCodes.Constant then
        return expr
    end

    if expr[3] == compiler.CFG.ExprOpCodes.Not then
        local rhs = expr[4]

        -- Remove double negations
        if rhs[3] == compiler.CFG.ExprOpCodes.Not then
            return compiler.FoldConstants( rhs[4] )
        end

        rhs = compiler.FoldConstants( rhs )
        if rhs[3] == compiler.CFG.ExprOpCodes.Constant then
            return { "bool", expr[2], compiler.CFG.ExprOpCodes.Constant, not rhs[4] }
        end
    elseif expr[3] == compiler.CFG.ExprOpCodes.Or then
        local lhs = compiler.FoldConstants( expr[4] )
        local rhs = compiler.FoldConstants( expr[5] )

        if  lhs[3] == compiler.CFG.ExprOpCodes.Constant and
            rhs[3] == compiler.CFG.ExprOpCodes.Constant then
            return { "bool", expr[2], compiler.CFG.ExprOpCodes.Constant, lhs[4] or rhs[4] }
        end

        if  lhs[3] == compiler.CFG.ExprOpCodes.Constant then
            return lhs[4] and { "bool", expr[2], compiler.CFG.ExprOpCodes.Constant, true } or rhs
        end

        if  rhs[3] == compiler.CFG.ExprOpCodes.Constant then
            return rhs[4] and { "bool", expr[2], compiler.CFG.ExprOpCodes.Constant, true } or lhs
        end
    elseif expr[3] == compiler.CFG.ExprOpCodes.And then
        local lhs = compiler.FoldConstants( expr[4] )
        local rhs = compiler.FoldConstants( expr[5] )

        if  lhs[3] == compiler.CFG.ExprOpCodes.Constant and
            rhs[3] == compiler.CFG.ExprOpCodes.Constant then
            return { "bool", expr[2], compiler.CFG.ExprOpCodes.Constant, lhs[4] and rhs[4] }
        end

        if  lhs[3] == compiler.CFG.ExprOpCodes.Constant then
            return lhs[4] and rhs or { "bool", expr[2], compiler.CFG.ExprOpCodes.Constant, false }
        end

        if  rhs[3] == compiler.CFG.ExprOpCodes.Constant then
            return rhs[4] and lhs or { "bool", expr[2], compiler.CFG.ExprOpCodes.Constant, false }
        end
    end

    return expr
end
