--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- bool literals and operators implementation.
--
local _, compiler = ...

local boolContext = { bool=1 }

function compiler.SetupBooleans( parser )

    parser:registerPrefixRule( "true", { compiler.SyntaxColor.Literal, completionContext = boolContext },
        function( p, _, tok )
            p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, true } )
            return true
        end
    )

    parser:registerPrefixRule( "false", { compiler.SyntaxColor.Literal, completionContext = boolContext },
        function( p, _, tok )
            p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
            return true
        end
    )

    parser:createPrefixOperatorRule( "not", compiler.Precedence.UnaryOps, { completionContext = boolContext },
        function( loc, rhsVal, p )
            if rhsVal[1] ~= "bool" then
                p.context:error( rhsVal[2], "Not: expected a boolean expression." )
                return true
            end

            loc = p.context.locs:createSpanningLocation( loc, rhsVal[2] )
            p:pushValue( compiler.FoldConstants( { "bool", loc, compiler.CFG.ExprOpCodes.Not, rhsVal } ) )
            return true
        end
    )

    parser:createLeftAssInfixOperatorRule( "or", compiler.Precedence.OrOp, { completionContext = boolContext },
        function( loc, lhsVal, rhsVal, p )
            if lhsVal[1] ~= "bool" then
                p.context:error( lhsVal[2], "Or: expected a boolean expression on the left hand side." )
                return true
            end

            if rhsVal[1] ~= "bool" then
                p.context:error( rhsVal[2], "Or: expected a boolean expression on the right hand side." )
                return true
            end

            loc = p.context.locs:createSpanningLocation( lhsVal[2], rhsVal[2] )
            p:pushValue( compiler.FoldConstants( { "bool", loc, compiler.CFG.ExprOpCodes.Or, lhsVal, rhsVal } ) )
            return true
        end
    )

    parser:createLeftAssInfixOperatorRule( "and", compiler.Precedence.AndOp, { completionContext = boolContext },
        function( loc, lhsVal, rhsVal, p )
            if lhsVal[1] ~= "bool" then
                p.context:error( lhsVal[2], "And: expected a boolean expression on the left hand side." )
                return true
            end

            if rhsVal[1] ~= "bool" then
                p.context:error( rhsVal[2], "And: expected a boolean expression on the right hand side." )
                return true
            end

            loc = p.context.locs:createSpanningLocation( lhsVal[2], rhsVal[2] )
            p:pushValue( compiler.FoldConstants( { "bool", loc, compiler.CFG.ExprOpCodes.And, lhsVal, rhsVal } ) )
            return true
        end
    )
end

compiler.CodeGen:RegisterExprOpcodeHandler( compiler.CFG.ExprOpCodes.Not, function( codegen, expr )
    return codegen:buildUnaryOp( expr, "not" )
end )

compiler.CodeGen:RegisterExprOpcodeHandler( compiler.CFG.ExprOpCodes.Or, function( codegen, expr )
    return codegen:buildBinaryOp( expr, "or" )
end )

compiler.CodeGen:RegisterExprOpcodeHandler( compiler.CFG.ExprOpCodes.And, function( codegen, expr )
    return codegen:buildBinaryOp( expr, "and" )
end )
