--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- Form conditionals
--
local _, compiler = ...

local prefixConditionalInfos = { compiler.SyntaxColor.Conditional, completionContext = { bool=1 } }

function compiler.SetupFormConditionals( parser )
    local _, playerClass = UnitClassBase( "player" )
    local druidColor = C_ClassColor.GetClassColor( "druid" ):GenerateHexColorMarkup()

    -- For priests, rogues, druids and shamans, "noform" means
    -- "not in any stance or shapeshit form, be it either stealth, shadowform or whatever".
    if playerClass == 4 or playerClass == 5 or playerClass == 11 or playerClass == 7 then
        parser:registerPrefixRule( "NoForm", prefixConditionalInfos,
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[noform]' )" } )
                return true
            end
        )
    else
        parser:registerPrefixRule( "NoForm", prefixConditionalInfos,
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, true } )
                return true
            end
        )
    end

    -- For druids and shamans only, we also have "Humanoid" as an alias for "noform".
    -- On all other classes, it's always true.
    if playerClass == 11 or playerClass == 7 then
        parser:registerPrefixRule( "Humanoid", prefixConditionalInfos,
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[noform]' )" } )
                return true
            end
        )
    else
        parser:registerPrefixRule( "Humanoid", prefixConditionalInfos,
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, true } )
                return true
            end
        )
    end

    -- Druid forms
    local _, _, _, druidColor = GetClassColor( 'DRUID' )
    druidColor = '|c' .. druidColor .. '|r'

    if playerClass == 11 then
        parser:registerPrefixRule( "Bear", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[form:1]' )" } )
                return true
            end
        )

        parser:registerPrefixRule( "Cat", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[form:2]' )" } )
                return true
            end
        )

        parser:registerPrefixRule( "Travel", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[form:3]' )" } )
                return true
            end
        )
    else
        -- If not a druid, they're all false constants
        parser:registerPrefixRule( "Bear", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )

        parser:registerPrefixRule( "Cat", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )

        parser:registerPrefixRule( "Travel", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )

        parser:registerPrefixRule( "Moonkin", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )

        parser:registerPrefixRule( "Treant", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )
    end

    -- Shaman form
    local _, _, _, shamanColor = GetClassColor( 'SHAMAN' )
    shamanColor = '|c' .. shamanColor .. '|r'

    if playerClass == 7 then
        parser:registerPrefixRule( "GhostWolf", { specificColor = shamanColor, class="shaman", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[form:1]' )" } )
                return true
            end
        )
    else
        -- If not a shaman, it's a false constant
        parser:registerPrefixRule( "GhostWolf", { specificColor = shamanColor, class="shaman", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )
    end


    if playerClass == 11 then
        parser:registerPrefixRule( "Moonkin", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:addInit( "local moonkin = self:GetParent():GetAttribute('moonkin')" )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional,
                    "moonkin and SecureCmdOptionParse( '[form:'..moonkin..']' )" } )
                return true
            end
        )

        parser:registerPrefixRule( "Treant", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:addInit( "local treant = self:GetParent():GetAttribute('treant')" )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional,
                    "treant and SecureCmdOptionParse( '[form:'..treant..']' )" } )
                return true
            end
        )

        parser:registerPrefixRule( "MountForm", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:addInit( "local mount = self:GetParent():GetAttribute('mount')" )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional,
                    "mount and SecureCmdOptionParse( '[form:'..mount..']' )" } )
                return true
            end
        )

        parser:registerPrefixRule( "TreeOfLife", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:addInit( "local tol = self:GetParent():GetAttribute('tol')" )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional,
                    "tol and SecureCmdOptionParse( '[form:'..tol..']' )" } )
                return true
            end
        )
    else
        parser:registerPrefixRule( "MountForm", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )

        parser:registerPrefixRule( "TreeOfLife", { specificColor = druidColor, class="druid", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )
    end

    -- Rogue forms
    local rogueColor = C_ClassColor.GetClassColor( "rogue" ):GenerateHexColorMarkup()

    if playerClass == 4 then
        -- The name is clumsy but it's to make it really distinct from the Stealthed conditional
        -- which is more generic (it corresponds to [stealth] in the stock macro system)
        parser:registerPrefixRule( "RogueStealth", { specificColor = rogueColor, class="rogue", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[form:1]' )" } )
                return true
            end
        )

        -- This corresponds to stance 2 which is vanish, subterfuge, shadow dance and shadowy duel aka
        -- "considered in stealth for most purposes but not necessarily actually stealthed" aka "what the fuck to even call this"
        parser:registerPrefixRule( "PseudoStealth", { specificColor = rogueColor, class="rogue", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[form:2]' )" } )
                return true
            end
        )

        -- Convenience shortcut for "I can use stealth abilities, regardless of the reason
        -- (actual stealth, subterfuge, shadow dance, shadowy duel)"
        parser:registerPrefixRule( "CanUseStealthSkills", { specificColor = rogueColor, class="rogue", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[form:1/2]' )" } )
                return true
            end
        )
    else
        -- If not a rogue, they're all false constants
        parser:registerPrefixRule( "RogueStealth", { specificColor = rogueColor, class="rogue", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )

        parser:registerPrefixRule( "PseudoStealth", { specificColor = rogueColor, class="rogue", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )

        parser:registerPrefixRule( "CanUseStealthSkills", { specificColor = rogueColor, class="rogue", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )
    end

    -- Priest forms
    local priestColor = C_ClassColor.GetClassColor( "priest" ):GenerateHexColorMarkup()

    if playerClass == 5 then
        parser:registerPrefixRule( "ShadowForm", { specificColor = priestColor, class="priest", completionContext = { bool=1 } },
            function( p, _, tok )
                p:addInit( "local spec = self:GetParent():GetAttribute('spec')" )
                local spec = { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "spec == 258" }
                local form = { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[form:1]' )" }
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.And, spec, form } )
                return true
            end
        )

        parser:registerPrefixRule( "SpiritOfRedemption", { specificColor = priestColor, class="priest", completionContext = { bool=1 } },
            function( p, _, tok )
                p:addInit( "local spec = self:GetParent():GetAttribute('spec')" )
                local spec = { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "spec == 257" }
                local form = { "bool", tok[2], compiler.CFG.ExprOpCodes.Conditional, "SecureCmdOptionParse( '[form:1]' )" }
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.And, spec, form } )
                return true
            end
        )
    else
        -- If not a priest, they're all false constants
        parser:registerPrefixRule( "ShadowForm", { specificColor = priestColor, class="priest", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )

        parser:registerPrefixRule( "SpiritOfRedemption", { specificColor = priestColor, class="priest", completionContext = { bool=1 } },
            function( p, _, tok )
                p:pushValue( { "bool", tok[2], compiler.CFG.ExprOpCodes.Constant, false } )
                return true
            end
        )
    end
end
