--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, compiler = ...
Smack_Compiler = compiler

local rep = string.rep

local defaultState = { 0, nil, true, 0, 0 }

function compiler.Indent( parserClass, textArray, currentLineHead, prevState, currentState, indentSize )
    if not currentState then
        return
    end

    prevState = prevState or defaultState
    local indentationHint = prevState[4]
    local currentIndention = prevState[5]
    local insideBlock = prevState[3]
    local empty = currentState[2]

    if #textArray == 1 then
        if textArray[1] == "{" then
            if empty and currentState[3] then
                currentIndention = currentIndention + indentSize
                currentIndention = currentIndention - ( currentIndention % indentSize )
            end

            local indention = rep( " ", currentIndention )

            if empty then
                textArray[1] = indention .. "{"
            else
                textArray[1] = "{"
            end

            textArray[2] = indention .. rep( " ", indentSize )
            textArray[3] = indention .. "}"
            return empty and 1 or nil, 2, currentIndention + indentSize + 1
        end

        if textArray[1] == "(" then
            textArray[1] = "()"
            return nil, 1, 2
        end
    end

    -- Nothing to do when inserting single line text (such as individual characters other than newlines)
    if #textArray == 1 then
        return
    end

    local context = compiler.Context:new()
    local tokenizer = compiler.Tokenizer:new( "", context, true )
    local ts = tokenizer:saveState()

    local blockCommentDepth = prevState[1]

    textArray[1] = currentLineHead .. textArray[1]

    for i=1,#textArray,1 do
        local sourceLine = strtrim( textArray[i] )

        if indentationHint > 0 then
            currentIndention = currentIndention + indentationHint * indentSize
        elseif indentationHint < 0 then
            currentIndention = currentIndention - 1
        end

        indentationHint = 0
        currentIndention = currentIndention - ( currentIndention % indentSize )

        tokenizer:restoreState( ts )
        tokenizer:setSource( sourceLine )
        tokenizer.blockCommentDepth = blockCommentDepth
        local tok = tokenizer:readToken()

        while tok do
            if tok[1] == compiler.TokenType.OpenBrace then
                if not insideBlock then
                    insideBlock = true
                    currentIndention = currentIndention - 1
                    currentIndention = currentIndention - ( currentIndention % indentSize )
                end
                    indentationHint = indentationHint + 1

            elseif tok[1] == compiler.TokenType.CloseBrace then
                currentIndention = currentIndention - 1
                currentIndention = currentIndention - ( currentIndention % indentSize )
                insideBlock = true
            else
                local infos = parserClass:getRuleInfos( tok )
                if infos then
                    if infos.shouldIndent then
                        indentationHint = indentationHint + 1
                        insideBlock = false
                    elseif infos.indentAsStatement then
                        if not insideBlock then
                            insideBlock = true
                            indentationHint = indentationHint - 1
                        end
                    end
                end
            end

            tok = tokenizer:readToken()
        end

        if i > 1 or empty then
            textArray[i] = rep( " ", currentIndention ) .. sourceLine
        end
        empty = false
        blockCommentDepth = tokenizer.blockCommentDepth
    end

    return 1, #textArray, strlenutf8(textArray[#textArray]) + 1
end
