--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- Variable scope management.
--
local _, compiler = ...

local ScopeStack = {}
compiler.ScopeStack = ScopeStack

function ScopeStack:new()
    local o = {}

    o.scopes = {}

    setmetatable( o, { __index = self } )
    return o
end

function ScopeStack:push()
    self.scopes[#self.scopes + 1] = {}
end

function ScopeStack:pop()
    self.scopes[#self.scopes] = nil
end

function ScopeStack:lookUp( name )
    local scopes = self.scopes
    for i=#scopes,1,-1 do
        local s=scopes[i]
        local var = s[name]
        if var ~= nil then
            return var
        end
    end
end

-- Add a variable declaration to the current scope.
-- returns true if successful.
-- It will fail if there is a variable currently visible
-- with the same name.
function ScopeStack:declVar( name, var )
    if self:lookUp( name ) ~= nil then
        return false
    end

    self.scopes[#self.scopes][name] = var
    return true
end
