--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- The else statement.
--
local _, compiler = ...

local ifContext = { pendingIf=1 }
local stmtContext = { stmt=1 }

function compiler.SetupElseStatement( parser )
    local infos =
    {
        compiler.SyntaxColor.ControlStmt,
        shouldIndent = true,
        completionContext = ifContext
    }

    parser:registerInfixRule( "else", infos,
        function( p, prec )
            local lhsVal = p.lhsVal
            if lhsVal and lhsVal[1] == "blockEnd" then
                return prec
            end

            return compiler.Precedence.Statement
        end,

        function( p, _, tok )
            local lhsVal = p.lhsVal
            if not lhsVal or lhsVal[1] ~= "pendingIf" then
                p.context:error( tok[2], "Unexpected 'else'." )
                return false
            end

            p:popValue()

            local ifHeaderBB = lhsVal.headerBB
            local contBB = ifHeaderBB.terminator[3]

            -- Create the else BB
            local elseBB = compiler.CFG.BasicBlock:new( p.cfg )
            p.cfg.currentInsertionPointBB = elseBB

            -- Parse the else
            p.scopeStack:push()
            local ok = p:parseExpr( compiler.Precedence.Statement, stmtContext )
            p.scopeStack:pop()

            if not ok then
                p.context:poison()
                return false
            end

            -- Retrieve the last block inserted as the else
            local elseLastBB = p.cfg.currentInsertionPointBB

            -- Wire up the else block into the if
            ifHeaderBB.terminator[3] = elseBB

            if elseLastBB and not elseLastBB.terminator then
                elseLastBB.terminator = { compiler.CFG.TerminatorType.Branch, contBB }
            end

            p.cfg.currentInsertionPointBB = contBB
            return true
        end
    )
end
