--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- The if statement.
--
local _, compiler = ...

local boolContext = { bool=1 }
local stmtContext = { stmt=1 }

function compiler.SetupIfStatement( parser )
    compiler.createStatement( parser, "if", { compiler.SyntaxColor.ControlStmt, shouldIndent = true },
        function( p, _, tok )
            p.expectedExpressionType = "bool"
            local success = p:parseExpr( compiler.Precedence.Statement, boolContext )
            local condVal = p:popValue()
            p.expectedExpressionType = nil

            if not success or not condVal or condVal[1] ~= "bool" then
                local loc = tok[2]
                if condVal then
                    loc = condVal[2]
                end

                p.context:error( loc, "Boolean expression expected after 'if'." )
                return
            end

            -- Save the preceding block, we'll have to set its terminator.
            local headerBB = p.cfg.currentInsertionPointBB

            -- Create the body BB
            local bodyBB = compiler.CFG.BasicBlock:new( p.cfg )
            p.cfg.currentInsertionPointBB = bodyBB

            -- Parse the body
            p.scopeStack:push()
            local ok = p:parseExpr( compiler.Precedence.Statement, stmtContext )
            p.scopeStack:pop()

            if not ok then
                p.context:poison()
                return
            end

            -- Retrieve the last block inserted as the body
            local bodyLastBB = p.cfg.currentInsertionPointBB

            -- Create the continuation block and set it as the new insertion point
            -- from now on.
            local contBB = compiler.CFG.BasicBlock:new( p.cfg )
            p.cfg.currentInsertionPointBB = contBB

            -- Wire everything up.
            headerBB.terminator = { compiler.CFG.TerminatorType.CondBranch, bodyBB, contBB, condVal }

            if bodyLastBB and not bodyLastBB.terminator then
                bodyLastBB.terminator = { compiler.CFG.TerminatorType.Branch, contBB }
            end

            p:pushValue( { "pendingIf", tok[2], headerBB = headerBB } )
            return true
        end
    )
end
