--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- Helpers to help reduce statement creation boilerplate
--
local _, compiler = ...

function compiler.createStatement( parser, name, infos, func )
    infos.completionContext = { stmt=1 }
    parser:registerPrefixRule( name, infos,
        function( p, prec, tok )
            if p.cfg.currentInsertionPointBB == nil then
                p.context:error( tok[2], "Unreachable code." )
                p.context:poison()
                return true
            end

            return func( p, prec, tok )
        end )
end

function compiler.createSimpleSlashCommandStatement( parser, name )
    compiler.createStatement( parser, name, { compiler.SyntaxColor.ActionStmt, indentAsStatement = true },
        function( p )
            p.cfg.currentInsertionPointBB:append( { compiler.CFG.OpCodes.SimpleSlashCmd, cmd = name } )
            return true
        end )
end

compiler.CodeGen:RegisterOpcodeHandler( compiler.CFG.OpCodes.SimpleSlashCmd, function( _, instr )
    return "m=m..'/" .. instr.cmd .. "\\n'"
end )
