--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, compiler = ...
local Locations = {}
compiler.Locations = Locations

local min = math.min
local max = math.max

function Locations:new()
    local o = {}
    setmetatable( o, { __index = self } )
    return o
end

function Locations:create( ... )
    self[#self+1] = { ... }
    return #self
end

function Locations:createSpanningLocation( l1, l2 )
    if l1 == nil or l2 == nil then
        return nil
    end

    local loc1 = self[l1]
    local loc2 = self[l2]

    local firstLine, firstCol, lastLine, lastCol

    if loc1[1] < loc2[1] then
        firstLine = loc1[1]
        firstCol = loc1[2]
    elseif loc1[1] > loc2[1] then
        firstLine = loc2[1]
        firstCol = loc2[2]
    else
        firstLine = loc1[1]
        firstCol = min( loc1[2], loc2[2] )
    end

    if loc1[3] > loc2[3] then
        lastLine = loc1[3]
        lastCol = loc1[4]
    elseif loc1[3] < loc2[3] then
        lastLine = loc2[3]
        lastCol = loc2[4]
    else
        lastLine = loc1[3]
        lastCol = max( loc1[4], loc2[4] )
    end

    return self:create( firstLine, firstCol, lastLine, lastCol )
end
