--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- The macro compiler front-end.
--
local _, compiler = ...
Smack_Compiler = compiler

local prologue = "local m = ''"
local epilogue = "self:SetAttribute( 'macrotext', m )"

local stateMachinePrologue = "local s = self:GetAttribute( 'state' )"
local stateMachineEpilogue = "self:SetAttribute( 'state', s )"

function compiler.CompileMacro( source, callbacks, cursorLine, cursorCol )
    local context = compiler.Context:new( callbacks )
    local timeslicer = context.callbacks.timeslicer

    local p = compiler.SmackParser:new( source, context, cursorLine, cursorCol )
    p:parse()

    context:emitPendingCompletions()

    if context.failed then
        return
    end

    -- Process event handler CFGs
    for _,ehCfg in ipairs( p.eventHandlers ) do
        timeslicer()
        compiler.DeadCodeElim( timeslicer, ehCfg )
        compiler.ComputeDominators( timeslicer, ehCfg )
    end

    compiler.DeadCodeElim( timeslicer, p.cfg )

    compiler.SplitStates( context, p.cfg )
    if context.failed then
        return
    end

    compiler.ComputeDominators( timeslicer, p.cfg )

    compiler.CheckLoops( context, p.cfg )
    if context.failed then
        return
    end

    local cg = compiler.CodeGen:new( context )

    cg:emit( prologue )

    for k,_ in pairs( p.inits ) do
        cg:emit( k )
    end

    if #p.cfg.entryPoints > 1 then
        cg:emit( stateMachinePrologue )
    end

    -- Generate event handlers code
    for _,ehCfg in ipairs( p.eventHandlers ) do
        timeslicer()
        cg:generate( ehCfg, #p.cfg.entryPoints > 1 )
    end

    cg:generate( p.cfg )

    if #p.cfg.entryPoints > 1 then
        cg:emit( stateMachineEpilogue )
    end

    cg:emit( epilogue )

    return cg:getGeneratedCode()
end

function compiler.MacroColorizer( palette )
    return function( sourceLine, state )
        return compiler.Colorize( compiler.SmackParser, sourceLine, state, palette )
    end
end

function compiler.MacroIndenter()
    return function( textArray, currentLineHead, prevState, currentState, indentSize )
        return compiler.Indent( compiler.SmackParser, textArray, currentLineHead, prevState, currentState, indentSize )
    end
end
