--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, ui = ...
local zu = ui.zu

local MacroCompletionList = {}
ui.MacroCompletionList = MacroCompletionList
setmetatable( MacroCompletionList, { __index = zu.PopupClass } )

MacroCompletionList.style = "dropdown"
MacroCompletionList.channel = "completion"

local function onSelectionChanged( self, newSel )
    if not self.shown or not newSel then
        return
    end

    if not newSel.append then
        self.editor:setPrimaryCursorExtent( self.l, self.c, self.editor:getPrimaryCursorPosition() )
    end

    self:hide()
    self.editor:pasteText( newSel.name )
    self.editor:focus()
end

function MacroCompletionList:new( editor )
    local o = {}
    o.editor = editor

    o.model = ui.MacroCompletionModel:new( editor, self )
    o.controller = zu.ListSelectionController:new()
    o.controller.stickyHiglight = true

    zu.AddListener( o.controller.selectionChangedEvent, o, onSelectionChanged )

    o.content = zu.List
    {
        model = o.model,
        controller = o.controller,
        enableKeyboardNavigation = true,
        preferredDisplayedItems = 16,
        minimizeHeight = true
    }

    setmetatable( o, { __index = self } )
    return o
end

function MacroCompletionList:setProvider( provider )
    self.model:setProvider( provider )
end

function MacroCompletionList:show( l, c )
    self.l, self.c = l, c
    local x, y = self.editor:lineColToCoords( l, c )

    if not x or not y or self.model:count() == 0 then
        self:hide()
        return
    end

    -- If we have more elements to display than before, reset the
    -- popup's size: we don't want to to be stuck at minsize when, for instance,
    -- transitioning from completing statements to completing conditions.
    -- Also reset item highlight, so that we avoid taking over the enter key
    -- more often than necessary.
    if self.lastCount and self.model:count() > self.lastCount then
        self:resetSize()
        self.controller:highlightElement( 0 )
    end

    self.lastCount = self.model:count()
    zu.PopupClass.show( self, x, y, self.editor.view.frame )
    self.editor:attachPopup( self )
end

function MacroCompletionList:hide()
    self.editor:detachPopup( self )
    zu.PopupClass.hide( self )
    self.controller:highlightElement( 0 )
end
