--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, ui = ...
local zu = ui.zu

local MacroCompletionItemClass = {}
setmetatable( MacroCompletionItemClass, { __index = zu.SelectableItemClass } )

local function colorize( self, name )
    local compiler = _G["Smack_Compiler"]
    return compiler.MacroColorizer( self:getSkin().syntaxColorPalette )( name )
end

function MacroCompletionItemClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function MacroCompletionItemClass:create( parentFrame )
    zu.SelectableItemClass.create( self, parentFrame )

    if not self.content then
        self.content = zu.Label{ label = colorize( self, self.completionItem.name ), font = self.font }
        self.content.parent = self
        self.content:create( parentFrame )
    end

    if not self.docPopup then
        self.docPopup = zu.CreateWidget( ui.MacroDoc, self.editor )
        self.docPopup.skin = self:getSkin()
        self.docPopup:setText( self.completionItem.doc or "" )
    end
end

function MacroCompletionItemClass:reuse()
    self.docPopup = zu.CreateWidget( ui.MacroDoc, self.editor )
    self.docPopup.skin = self:getSkin()
    self.docPopup:setText( self.completionItem.doc or "" )
    zu.SelectableItemClass.reuse( self )
end

function MacroCompletionItemClass:release()
    if self.docPopup then
        zu.ReleaseWidget( self.docPopup )
        self.docPopup = nil
    end

    zu.SelectableItemClass.release( self )
end

function MacroCompletionItemClass:setItem( item )
    self.completionItem = item
    if self.content then
        self.content:setLabel( colorize( self, item.name ) )
        self.docPopup:setText( self.completionItem.doc or "" )
    end
end

function MacroCompletionItemClass:highlight( h )
    zu.SelectableItemClass.highlight( self, h )
    if h then
        if self.completionItem.doc then
            self.docPopup:show( self.content.width + zu.Scale( 1 ), 0, self.selFrame )
        end
        self.parentPopup.attachedPopup = self.docPopup
    else
        self.docPopup:hide()
    end
end

function MacroCompletionItemClass:hide()
    self.docPopup:hide()
    zu.SelectableItemClass.hide( self )
end

function MacroCompletionItemClass:getIndex()
    return self.model:getElementIndex( self.completionItem )
end

local MacroCompletionModel = {}
ui.MacroCompletionModel = MacroCompletionModel
setmetatable( MacroCompletionModel, { __index = zu.PopupClass } )

function MacroCompletionModel:new( editor, parentPopup )
    local o = {}
    o.editor = editor
    o.parentPopup = parentPopup
    o.indices = {}
    o.updateEvent = {}
    o.modelResetEvent = {}
    o.labelTpl = { font = editor.font, label = "W" }
    setmetatable( o, { __index = self } )
    return o
end

function MacroCompletionModel:create()
end

function MacroCompletionModel:release()
end

function MacroCompletionModel:setProvider( provider )
    self.provider = provider

    wipe( self.indices )
    for i=1,provider:count(),1 do
        self.indices[ provider:getItem( i ) ] = i
    end

    zu.Raise( self.modelResetEvent )
end

function MacroCompletionModel:count()
    return self.provider:count()
end

function MacroCompletionModel:getElementSize( view )
    local w = zu.CreateWidget( zu.LabelClass, self.labelTpl )
    w.parent = view
    w:create( view.frame )
    w:calcMinMax()

    local height = w.minHeight
    local levelRange = w.levelRange
    zu.ReleaseWidget( w )

    return height, 96, 128, nil, levelRange
end

function MacroCompletionModel:getElement( i )
    return self.provider:getItem( i )
end

function MacroCompletionModel:getElementIndex( e )
    return self.indices[e] or 0
end

function MacroCompletionModel:getWidget( i )
    local w = zu.CreateWidget( MacroCompletionItemClass )
    w.editor = self.editor
    w.model = self
    w.parentPopup = self.parentPopup
    w:setItem( self.provider:getItem( i ) )
    return w
end
