--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, ui = ...
local zu = ui.zu

local MacroDoc = {}
ui.MacroDoc = MacroDoc
setmetatable( MacroDoc, { __index = zu.PopupClass } )

MacroDoc.style = "doc"
MacroDoc.channel = "doc"

local function GetCompiler()
    if not IsAddOnLoaded( "Smack_Compiler" ) then
        EnableAddOn( "Smack_Compiler" )
        LoadAddOn( "Smack_Compiler" )
    end

    return _G["Smack_Compiler"]
end

function MacroDoc:new( editor )
    local o = {}

    o.editor = editor
    o.content = zu.CreateWidget( zu.RichTextClass, {} )
    setmetatable( o, { __index = self } )
    return o
end

function MacroDoc:reuse( editor )
    self.editor = editor
    self.content = zu.CreateWidget( zu.RichTextClass, {} )
    zu.PopupClass.reuse()
end

function MacroDoc:release()
    self.content:release()
    zu.PopupClass.release( self )
end

function MacroDoc:setText( text )
    self.content.skin = self.skin
    self.content.codeColorizer = GetCompiler().MacroColorizer( self.skin.syntaxColorPalette )
    self.content:setText( text )
end

function MacroDoc:show( x, y, frame, point )
    zu.PopupClass.show( self, x, y, frame, point )
    self.editor:attachPopup( self )
end

function MacroDoc:hide()
    self.editor:detachPopup( self )
    zu.PopupClass.hide( self )
end
