--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- A widget representing an item in the macro list.
--
local _, ui = ...
local zu = ui.zu

local MacroItemClass = {}
ui.MacroItemClass = MacroItemClass
setmetatable( MacroItemClass, { __index = zu.SelectableItemClass } )

function MacroItemClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

local function update( self )
    if self.content and self.macro then
        local name = self.macro.name

        local smack = _G["Smack_Main"]
        local ti = smack.SpecialTagsCheckForSpecificClass( self.macro.tags )
        if ti then
            name = ti.colorTag .. name
        end

        if self.macro.hasErrors then
            -- Just prepend a red ! for macros that don't compile
            -- Maybe we could do something fancier like a small sign icon
            -- but this will have to wait until the "make the UI not
            -- look like ass" development phase
            name = "|cFFFF0000!|r " .. name
        end

        self.content:setAlpha( self.macro.properties.enabled and 1 or 0.6 )
        self.content:setLabel( name )
    end
end

function MacroItemClass:release()
    if self.macro then
        zu.RemoveListener( self.macro.updateEvent, self )
        self.macro = nil
    end
    zu.SelectableItemClass.release( self )
end

function MacroItemClass:create( parentFrame )
    zu.SelectableItemClass.create( self, parentFrame )

    if not self.content then
        self.content = zu.Label { minWidthOverride = 80 }
        self.content.parent = self
        self.content:create( parentFrame )
    end

    self:setMacro( self.macro )
end

function MacroItemClass:setMacro( macro )
    if macro == nil then
        return
    end

    if self.macro then
        zu.RemoveListener( macro.updateEvent, self )
        zu.RemoveListener( macro.tags.tagAddedEvent, self )
        zu.RemoveListener( macro.tags.tagRemovedEvent, self )
        zu.RemoveListener( macro.tags.tagsUpdatedEvent, self )
    end

    self.macro = macro

    update( self )
    zu.AddListener( macro.updateEvent, self, update )
    zu.AddListener( macro.tags.tagAddedEvent, self, update )
    zu.AddListener( macro.tags.tagRemovedEvent, self, update )
    zu.AddListener( macro.tags.tagsUpdatedEvent, self, update )
end

function MacroItemClass:getIndex()
    return self.model:getElementIndex( self.macro )
end
