--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, ui = ...
local zu = ui.zu
Smack_UI = ui

SmackUISettings = SmackUISettings or {}

local macroListController = zu.ListSelectionController:new()
local macroListModel = ui.MacroListModel:new()
local macroEditor = ui.MacroEditor { listController = macroListController }

local statusBar = zu.Hider
{
    content = zu.Warning {},
    contentHidden = true
}

local pager = zu.Pager
{
    ui.AboutPane(),
    macroEditor
}

local window = zu.Window
{
    name = "MainWindow",
    title = "Smack",
    onAbout = function()
        -- Clear the selection, which will cause the about pane to be displayed
        macroListController:selectElement( 0 )
    end,

    zu.HorResizeGroup
    {
        name = "TopLevelResizeGroup",

        zu.VerticalLayout
        {
            ui.MacroList( macroListModel, macroListController ),

            zu.Button
            {
                type = "action",
                label = "New macro",
                onClick = function()
                    macroListController:createElement( "New macro" )
                    macroEditor:selectName()
                end
            }
        },

        zu.VerticalLayout
        {
            pager,
            statusBar
        }
    }
}

zu.AddListener( macroListController.selectionChangedEvent, "Smack_UI",
    function( _, selected, isNew )
        if selected then
            pager:switch( 2 )
            macroEditor:setMacro( selected, isNew )
        else
            -- Whenever we don't have a selection, display the about pane instead of the editor
            pager:switch( 1 )
        end
    end )

local smack = _G["Smack_Main"]

local function displayPendingChangesNotice()
    statusBar.content:setText( "Changes are pending and will be applied once you're out of combat." )
    statusBar:showContent()
end

zu.AddListener( smack.DeferedUpdatesQueuedEvent, "Smack_UI",
    function()
        displayPendingChangesNotice()
    end )

zu.AddListener( smack.DeferedUpdatesFlushedEvent, "Smack_UI",
    function()
        statusBar:hideContent()
        statusBar.content:setText( "" )
    end )

function ui.Show()
    zu.Settings = SmackUISettings

    if smack.AreUpdatesPending() then
        displayPendingChangesNotice()
    end

    window:show()
end

function ui.Hide()
    window:hide()
end
