--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- Base class for widgets built out of multiple
-- elementary widgets.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local CompositeClass = {}
zu.CompositeClass = CompositeClass
setmetatable( CompositeClass, { __index = zu.Widget } )

function CompositeClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function CompositeClass:release()
    if self.content then
        self.content:release()
    end
end

function CompositeClass:reuse( ... )
    if self.content then
        self.content:reuse( ... )
    end
end

function CompositeClass:calcMinMax()
    self.content:calcMinMax()

    self.minWidth = self.content.minWidth
    self.maxWidth = self.content.maxWidth
    self.preferredWidth = self.content.preferredWidth

    self.minHeight = self.content.minHeight
    self.maxHeight = self.content.maxHeight
    self.preferredHeight = self.content.preferredHeight

    self.levelRange = self.content.levelRange
end

function CompositeClass:heightForWidth( width )
    local hfw = self.content:heightForWidth( width )
    self.hfw = hfw
    return hfw
end

function CompositeClass:setLevel( level )
    self.content:setLevel( level )
end

function CompositeClass:layout( x, y, width, height )
    zu.Widget.layout( self, x, y, width, height )
    self.content:layout( x, y, width, height )
end

function CompositeClass:show()
    if self.content then
        self.content:show()
    end
end

function CompositeClass:hide()
    if self.content then
        self.content:hide()
    end
end

function CompositeClass:freeze()
    zu.Widget.freeze( self )

    if self.content then
        self.content:freeze()
    end
end

function CompositeClass:unfreeze()
    zu.Widget.unfreeze( self )

    if self.content then
        self.content:unfreeze()
    end
end
