--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local ButtonClass = {}
zu.ButtonClass = ButtonClass
setmetatable( ButtonClass, { __index = zu.Widget } )

ButtonClass.fontObject = "GameFontNormalSmall"
ButtonClass.innerMargin = 8

function ButtonClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.Button( b )
    return ButtonClass:new( b )
end

function ButtonClass:create( parentFrame )
    if not self.decorator then
        self.decorator = self:getSkin().ButtonDecorator:new( self )
    end

    if self.frame then
        self.frame:ClearAllPoints()
        self.frame:SetParent( parentFrame )
    else
        self.frame = CreateFrame( "Button", nil, parentFrame )
    end

    if self.onClick then
        self.frame:SetScript( "OnClick", function( _, ... )
            self.onClick( self, ... )
        end )
    else
        self.frame:SetScript( "OnClick", nil )
    end

    self.decorator:apply()
end

function ButtonClass:setLabel( label )
    self.decorator:setLabel( label )
    self:relayout()
end

function ButtonClass:calcMinMax()
    self.decorator:calcMinMax()
end

function ButtonClass:heightForWidth( width )
    self.decorator:heightForWidth( width )
end

function ButtonClass:setLevel( level )
    zu.Widget.setLevel( self, level )
    self.decorator:setLevel( level )
end

function ButtonClass:layout( ... )
    zu.Widget.layout( self, ... )
    self.decorator:layout( ... )
end

function ButtonClass:disable()
    self.frame:Disable()
    self.decorator:update()
end

function ButtonClass:enable()
    self.frame:Enable()
    self.decorator:update()
end
