--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local CheckBoxClass = {}
zu.CheckBoxClass = CheckBoxClass
setmetatable( CheckBoxClass, { __index = zu.Widget } )

CheckBoxClass.fontObject = "GameFontNormalSmall"
CheckBoxClass.innerMargin = 8

function CheckBoxClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.CheckBox( tpl )
    return CheckBoxClass:new( tpl )
end

function CheckBoxClass:create( parentFrame )
    if not self.decorator then
        self.decorator = self:getSkin().CheckBoxDecorator:new( self )
    end

    if self.frame then
        self.frame:ClearAllPoints()
        self.frame:SetParent( parentFrame )
    else
        self.frame = CreateFrame( "CheckButton", nil, parentFrame )
    end

    if self.onClick then
        self.frame:SetScript( "OnClick", function( frame )
            self.checked = frame:GetChecked()
            self.settings.checked = self.checked
            self.decorator:update()
            self.onClick( frame:GetChecked() )
        end )
    else
        self.frame:SetScript( "OnClick", nil )
    end

    self.settings = {}
    if self.name then
        self.settings = self:getWindow().settings[self.name] or {}
        self:getWindow().settings[self.name] = self.settings
    end

    if self.settings.checked ~= nil then
        local update = self.checked ~= self.settings.checked
        self.checked = self.settings.checked
        if update then
            self.onClick( self.checked )
        end
    end

    self.frame:SetChecked( self.checked )

    self.decorator:apply()
end

function CheckBoxClass:setLabel( label )
    self.label = label
    self.decorator:update()
end

function CheckBoxClass:setChecked( checked )
    self.checked = checked
    self.frame:SetChecked( checked )
    self.decorator:update()
end

function CheckBoxClass:calcMinMax()
    self.decorator:calcMinMax()
end

function CheckBoxClass:heightForWidth( width )
    self.decorator:heightForWidth( width )
    self.hfw = self.decorator.hfw
    return self.hfw
end

function CheckBoxClass:setLevel( level )
    zu.Widget.setLevel( self, level )
    self.decorator:setLevel( level )
end

function CheckBoxClass:layout( x, y, width, height )
    zu.Widget.layout( self, x, y, width, height )

    self.frame:SetSize( width, height )
    self.frame:SetPoint( "TOPLEFT", self.frame:GetParent(), "TOPLEFT", x, -y )

    self.decorator:layout( 0, 0, width, height )
end
