--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local DialogClass = {}
zu.DialogClass = DialogClass
setmetatable( DialogClass, { __index = zu.CompositeClass } )

DialogClass.okLabel = "Ok"
DialogClass.cancelLabel = "Cancel"

function DialogClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.Dialog( tpl )
    return DialogClass:new( tpl )
end

function DialogClass:setText( text )
    self.text = text
    if self.textWidget then
        self.textWidget:setText( text )
    end
end

function DialogClass:create( parentFrame )
    if not self.content then
        self.textWidget = zu.Text { text = self.text }
        self.okButton = self.onOk and zu.Button { type = "action", label = self.okLabel } or nil
        self.cancelButton = self.onCancel and zu.Button { type = "action", label = self.cancelLabel } or nil

        if self.okButton then
            self.okButton.onClick = self.onOk
        end

        if self.cancelButton then
            self.cancelButton.onClick = self.onCancel
        end

        local buttons
        if self.okButton and self.cancelButton then
            buttons = zu.HorizontalLayout { self.cancelButton, self.okButton }
        elseif self.okButton then
            buttons = self.okButton
        elseif self.cancelButton then
            buttons = self.cancelButton
        end

        local innerSection
        if buttons then
            innerSection = zu.VerticalLayout
            {
                self.textWidget,
                buttons
            }
        else
            innerSection = self.textWidget
        end

        self.content = zu.HorizontalLayout
        {
            zu.Spacer {}, innerSection, zu.Spacer {}
        }

        self.content.parent = self
    end

    self.content:create( parentFrame )
end
