--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local EditBoxClass = {}
zu.EditBoxClass = EditBoxClass
setmetatable( EditBoxClass, { __index = zu.Widget } )

-- Default values, can be overriden by the template or inside individual instances
EditBoxClass.text = ""

function EditBoxClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.EditBox( l )
    return EditBoxClass:new( l )
end

function EditBoxClass:create( parentFrame )
    if not self.decorator then
        self.decorator = self:getSkin().EditBoxDecorator:new( self )
    end

    if self.frame then
        self.frame:ClearAllPoints()
        self.frame:SetParent( parentFrame )
    else
        self.frame = CreateFrame( "Frame", nil, parentFrame )

        self.editBox = CreateFrame( "EditBox", nil, self.frame )
        self.editBox:SetAutoFocus( false )

        if self.prompt then
            self.promptFrame = self.editBox:CreateFontString( nil, "ARTWORK" )
            self.promptFrame:SetAllPoints()
            self.promptFrame:SetJustifyH( "LEFT" )
            self.promptFrame:SetWordWrap( false )
        end

        self.editBox:SetScript( "OnEnterPressed", function( self )
            self:ClearFocus()
        end )

        self.editBox:SetScript( "OnTextChanged", function()
            local newText = self.editBox:GetText()
            if newText == self.text then
                return
            end

            self.textChanged = true
            self.text = self.editBox:GetText()

            if self.onTextChanged then
                self.onTextChanged( self.editBox:GetText() )
            end
        end )

        self.editBox:SetScript( "OnEditFocusGained", function()
            if self.promptFrame then
                self.promptFrame:Hide()
            end
            end )

        self.editBox:SetScript( "OnEditFocusLost", function( f )
            f:HighlightText( 0, 0 )
            self.text = self.editBox:GetText()

            if self.textChanged and self.onTextEditionFinished then
                self.onTextEditionFinished( self.editBox:GetText() )
            end

            if self.promptFrame then
                self.promptFrame:SetShown( self.editBox:GetText() == "" )
            end

            self.textChanged = false
        end )

        self.decorator:apply()

        if self.promptFrame then
            self.promptFrame:SetText( self.prompt )
        end
    end

    if self.editBox:GetText() ~= self.text then
        self.editBox:SetText( self.text )
    end
end

function EditBoxClass:setText( text )
    self.text = text
    self.editBox:SetText( text )
    self:calcMinMax()
end

function EditBoxClass:select()
    self.editBox:HighlightText()
    self.editBox:SetFocus()
end

function EditBoxClass:calcMinMax()
    self.decorator:calcMinMax()
end

function EditBoxClass:heightForWidth( width )
    self.decorator:heightForWidth( width )
end

function EditBoxClass:setLevel( level )
    zu.Widget.setLevel( self, level )
    self.decorator:setLevel( level )
end

function EditBoxClass:layout( ... )
    zu.Widget.layout( self, ... )
    self.decorator:layout( ... )
end
