--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local FillerClass = {}
zu.FillerClass = FillerClass
setmetatable( FillerClass, { __index = zu.SpacerClass } )

FillerClass.levelRange = 1

function FillerClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.Filler( tpl )
    return FillerClass:new( tpl )
end

function FillerClass:create( parentFrame )
    if not self.frame then
        self.frame = CreateFrame( "Frame", nil, parentFrame )

        if self[4] then
            local tex = self.frame:CreateTexture( nil, "BACKGROUND" )
            tex:SetAllPoints( self.frame )
            tex:SetDrawLayer( "BACKGROUND" )
            tex:SetColorTexture( self[1], self[2], self[3], self[4] )
            self.frame:EnableMouse( self.shouldEatMouseEvents )
        end
    end

    if self.onHover then
        self.frame:SetScript( "OnEnter", function()
            self.onHover( true )
        end )

        self.frame:SetScript( "OnLeave", function()
            self.onHover( false )
        end )
    else
        self.frame:SetScript( "OnEnter", nil )
        self.frame:SetScript( "OnLeave", nil )
    end
end

function FillerClass:show()
    if self.frame then
        self.frame:Show()
    end
end

function FillerClass:hide()
    if self.frame then
        self.frame:Hide()
    end
end

function FillerClass:setLevel( level )
    zu.SpacerClass.setLevel( self, level )
    self.frame:SetFrameLevel( level )
end

function FillerClass:layout( x, y, width, height )
    zu.Widget.layout( self, x, y, width, height )
    self.frame:SetSize( width, height )
    self.frame:SetPoint( "TOPLEFT", self.frame:GetParent(), "TOPLEFT", x, -y )
end

function FillerClass:freeze()
    zu.Widget.freeze( self )
    self.frozen = true
    if self.onHover then
        self.onHover( false )
    end
end

function FillerClass:unfreeze()
    zu.Widget.unfreeze( self )
    self.frozen = false
end
