--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local IconClass = {}
zu.IconClass = IconClass
setmetatable( IconClass, { __index = zu.Widget } )
IconClass.levelRange = 1

IconClass.color = { 1, 1, 1, 1 }

function IconClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.Icon( l )
    return IconClass:new( l )
end

local function update( self )
    if self.hover then
        self.tex:SetVertexColor( unpack( self.highlightColor ) )
    else
        self.tex:SetVertexColor( unpack( self.color ) )
    end
end

function IconClass:create( parentFrame )
    if self.frame then
        self.frame:ClearAllPoints()
        self.frame:SetParent( parentFrame )
    else
        self.frame = CreateFrame( "Frame", nil, parentFrame )
    end

    if not self.tex then
        local tex, width, height = self:getSkin():createIconTexture( self.frame, self.icon, self.flipX, self.flipY, self.rotateRight90 )
        self.tex = tex
        self.iconWidth = width
        self.iconHeight = height
    end

    self.tex:SetVertexColor( unpack( self.color ) )

    if self.highlightColor then
        self.frame:SetScript( "OnEnter", function()
            self.hover = true
            update( self )
        end )

        self.frame:SetScript( "OnLeave", function()
            self.hover = false
            update( self )
        end )
    else
        self.frame:SetScript( "OnEnter", nil )
        self.frame:SetScript( "OnLeave", nil )
    end
end

function IconClass:calcMinMax()
    local w = zu.Scale( self.iconWidth )
    local h = zu.Scale( self.iconHeight )

    self.minWidth = w
    self.maxWidth = w
    self.preferredWidth = w
    self.minHeight = h
    self.maxHeight = h
    self.preferredHeight = h
end

function IconClass:setLevel( level )
    zu.Widget.setLevel( self, level )
    self.frame:SetFrameLevel( level )
end

function IconClass:layout( x, y, width, height )
    zu.Widget.layout( self, x, y, width, height )
    self.frame:SetSize( width, height )
    self.frame:SetPoint( "TOPLEFT", self.frame:GetParent(), "TOPLEFT", x, -y )
end

function IconClass:setIcon( icon )
    self.icon = icon
    if self.tex then
        local _, width, height = self:getSkin():createIconTexture( self.frame, self.icon, self.flipX, self.flipY, self.rotateRight90, self.tex )
        self.iconWidth = width
        self.iconHeight = height
    end
end

function IconClass:freeze()
    zu.Widget.freeze( self )
    self.frozen = true
    update( self )
end

function IconClass:unfreeze()
    zu.Widget.unfreeze( self )
    self.frozen = false
    update( self )
end
