--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local LabelClass = {}
zu.LabelClass = LabelClass
setmetatable( LabelClass, { __index = zu.Widget } )

-- Default values, can be overriden by the template or inside individual instances
LabelClass.label = ""
LabelClass.justify = "LEFT"
LabelClass.alpha = 1

function LabelClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.Label( l )
    return LabelClass:new( l )
end

function LabelClass:create( parentFrame )
    if not self.decorator then
        self.decorator = self:getSkin().LabelDecorator:new( self )
    end

    if self.frame then
        self.frame:ClearAllPoints()
        self.frame:SetParent( parentFrame )
    else
        self.frame = CreateFrame( "Frame", nil, parentFrame )
    end

    if not self.text then
        self.text = self.frame:CreateFontString( nil, "ARTWORK" )
        self.text:SetAllPoints()
        self.text:SetJustifyH( self.justify )
        self.text:SetWordWrap( false )
        self.text:SetAlpha( self.alpha )
    end

    self.text:SetFontObject( self.font or self.decorator.font )
    self.decorator:apply()
    self.text:SetText( self.label )
end

function LabelClass:setLabel( label )
    self.label = label
    if self.text then
        self.text:SetText( label )
        self:relayout()
    end
end

function LabelClass:setAlpha( alpha )
    self.alpha = alpha
    if self.text then
        self.text:SetAlpha( alpha )
    end
end

function LabelClass:calcMinMax()
    self.decorator:calcMinMax()
end

function LabelClass:heightForWidth( width )
    self.decorator:heightForWidth( width )
end

function LabelClass:setLevel( level )
    zu.Widget.setLevel( self, level )
    self.decorator:setLevel( level )
end

function LabelClass:layout( ... )
    zu.Widget.layout( self, ... )
    self.decorator:layout( ... )
end
