--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local ceil = math.ceil

local TextClass = {}
zu.TextClass = TextClass
setmetatable( TextClass, { __index = zu.Widget } )

-- Default values, can be overriden by the template or inside individual instances
TextClass.text = ""
TextClass.justify = "CENTER"

-- We can't simply use the widget's fontstring to do the measurement for
-- heightForWidth: it doesn't return the correct size on the first frame
-- after it's created. So here's a shitty workaround where we use a global,
-- hidden fontstring created in advance for that purpose.
local fontStringHelper = UIParent:CreateFontString()
fontStringHelper:Hide()
fontStringHelper:SetScale( zu.ContentScaleFactor() )
fontStringHelper:SetWordWrap( true )
fontStringHelper:SetNonSpaceWrap( true )
fontStringHelper:SetHeight( 10000 )

function TextClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.Text( tpl )
    return TextClass:new( tpl )
end

function TextClass:reuse( tpl )
    self:setText( tpl.text )
    zu.Widget.reuse( self )
end

function TextClass:release()
    if self.frame then
        self.textFrame:SetText( "" )
        self.frame:Hide()
        self.frame:ClearAllPoints()
        self.frame:SetParent( UIParent )
    end
end

function TextClass:create( parentFrame )
    if not self.decorator then
        self.decorator = self:getSkin().TextDecorator:new( self )
    end

    if self.frame then
        self.frame:ClearAllPoints()
        self.frame:SetParent( parentFrame )
    else
        self.frame = CreateFrame( "Frame", nil, parentFrame )
   end

    if not self.textFrame then
        self.textFrame = self.frame:CreateFontString( nil, "ARTWORK" )
        self.textFrame:SetAllPoints()
        self.textFrame:SetJustifyH( self.justify )
        self.textFrame:SetWordWrap( true )
        self.textFrame:SetNonSpaceWrap( true )
    end

    self.decorator:apply()
    self.textFrame:SetText( self.text )
end

function TextClass:setText( text )
    self.text = text

    if self.textFrame then
        self.textFrame:SetText( text )
        self:relayout()
    end
end

function TextClass:calcMinMax()
    self.decorator:calcMinMax()
end

function TextClass:heightForWidth( width )
    fontStringHelper:SetFontObject( self.decorator:getFontObject() )
    fontStringHelper:SetText( self.text )
    fontStringHelper:SetWidth( ceil( width ) )
    self.hfw = ceil( fontStringHelper:GetStringHeight() )
    return self.hfw
end

function TextClass:setLevel( level )
    zu.Widget.setLevel( self, level )
    self.frame:SetFrameLevel( level )
end

function TextClass:layout( x, y, width, height )
    zu.Widget.layout( self, x, y, width, height )
    self.frame:SetSize( ceil( width ), ceil( height ) )
    self.frame:SetPoint( "TOPLEFT", self.frame:GetParent(), "TOPLEFT", x, -y )
end
