--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local HRuleDecorationClass = {}
zu.HRuleDecorationClass = HRuleDecorationClass
setmetatable( HRuleDecorationClass, { __index = zu.Widget } )

HRuleDecorationClass.r = 1
HRuleDecorationClass.g = 1
HRuleDecorationClass.b = 1
HRuleDecorationClass.a = 1

HRuleDecorationClass.firstCol = 1
HRuleDecorationClass.lastCol = 1
HRuleDecorationClass.bottomInset = 7

HRuleDecorationClass.thickness = 1

function HRuleDecorationClass:new( tpl )
    local o = {}
    o.frames = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )

    if o.above then
        o.bottomInset = 0
        o.topInset = 8
    else
        o.bottomInset =  8
        o.topInset = 0
    end

    return o
end

function HRuleDecorationClass:reuse()
    if self.above then
        self.bottomInset = 0
        self.topInset = 8
    else
        self.bottomInset = 8
        self.topInset = 0
    end
end

function HRuleDecorationClass:release()
    local frames = self.frames
    for _,v in pairs( frames ) do
        zu.ReleaseFrame( "hrule", v )
    end

    wipe( frames )
end

function HRuleDecorationClass:setLine( lineNum )
    self.firstLine = lineNum
    self.lastLine = lineNum
end

function HRuleDecorationClass:line()
    return self.firstLine
end

function HRuleDecorationClass:attach( lineWidget )
    local frame = self.frames[lineWidget] or zu.CreateFrame( "hrule", "Frame", lineWidget.frame )
    self.frames[lineWidget] = frame

    if not frame.inited then
        frame.tex = frame:CreateTexture( nil, "BACKGROUND" )
        frame.tex:SetAllPoints( frame )
        frame.inited = true
    end

    frame.tex:SetColorTexture( self.r, self.g, self.b, self.a )
    self:update( lineWidget )
    frame:Show()
end

function HRuleDecorationClass:update( lineWidget )
    local frame = self.frames[lineWidget]
    if not frame then
        return
    end

    local offset = floor( ( zu.Scale( 8 ) + zu.Scale( self.thickness ) ) * 0.5 )

    if self.above then
        frame:SetPoint( "TOPLEFT", lineWidget.frame, "TOPLEFT", 0, offset )
        frame:SetPoint( "TOPRIGHT", lineWidget.frame, "TOPRIGHT", 0, offset )
    else
        frame:SetPoint( "BOTTOMLEFT", lineWidget.frame, "BOTTOMLEFT", 0, -offset )
        frame:SetPoint( "BOTTOMRIGHT", lineWidget.frame, "BOTTOMRIGHT", 0, -offset )
    end

    frame:SetHeight( zu.Scale( self.thickness ) )
end

function HRuleDecorationClass:detach( lineWidget )
    if self.frames[lineWidget] then
        zu.ReleaseFrame( "hrule", self.frames[lineWidget] )
        self.frames[lineWidget] = nil
    end
end

function HRuleDecorationClass:hitTest()
    return false
end

function HRuleDecorationClass:hovering()
end
