--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- Uses the text editor components to display static, copiable text
-- with syntax coloring, no scrolling, and no word wrapping. Used by the
-- rich text widget to display code blocks.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local InlineTextViewClass = {}
zu.InlineTextViewClass = InlineTextViewClass
setmetatable( InlineTextViewClass, { __index = zu.CompositeClass } )

-- Default values, can be overriden by the template or inside individual instances
InlineTextViewClass.text = ""

local defaultFont = CreateFont( "zuDefaultEditorFont" )
defaultFont:SetFont( "Interface\\AddOns\\Smack_UI\\zlui\\media\\Hack-Regular.ttf", 16, "" )
InlineTextViewClass.font = defaultFont

function InlineTextViewClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.InlineTextView( s )
    return InlineTextViewClass:new( s )
end

function InlineTextViewClass:create( parentFrame )
    if not self.model then
        self.model = zu.EditorModel:new()
        self.model.colorizer = self.colorizer
        self.font:SetTextColor( self:getSkin():getEditorTextColor() )
        self.model:setTextBuffer( self.text )

        self.view = zu.EditorView {}
        self.view.readOnly = true
        self.view.canScroll = false
        self.view.model = self.model
        self.view.font = self.font

        self.controller = zu.EditorController:new( self.model, true )
        self.controller.doubleClickSelectsAll = self.doubleClickSelectsAll

        self.content = self.view
        self.content.parent = self
        self.content:create( parentFrame )

        self.controller:setView( self.view )
    end
end

function InlineTextViewClass:calcMinMax()
    self.view:calcMinMax()

    local width = self.model.btree:width() * self.view.charWidthScale
    local height = self.model.btree:height() * self.view.lineHeightScale

    self.minWidth = width
    self.maxWidth = width
    self.preferredWidth = width

    self.minHeight = height
    self.maxHeight = height
    self.preferredHeight = height

    self.levelRange = 1
end

function InlineTextViewClass:show()
    zu.CompositeClass.show( self )
    self.controller:unfreeze()
end

function InlineTextViewClass:hide()
    self.controller:freeze()
    zu.CompositeClass.hide( self )
end

function InlineTextViewClass:freeze()
    zu.CompositeClass.freeze( self )
    self.controller:freeze()
end

function InlineTextViewClass:unfreeze()
    zu.CompositeClass.unfreeze( self )
    self.controller:unfreeze()
end

function InlineTextViewClass:setText( text )
    self.text = text
    if self.controller then
        self.controller:setText( text )
    end
end
