--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local SquiggleDecorationClass = {}
zu.SquiggleDecorationClass = SquiggleDecorationClass
setmetatable( SquiggleDecorationClass, { __index = zu.Widget } )

SquiggleDecorationClass.r = 1
SquiggleDecorationClass.g = 0
SquiggleDecorationClass.b = 0
SquiggleDecorationClass.a = 1

function SquiggleDecorationClass:new( tpl )
    local o = {}
    o.frames = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )

    if o.popupContent then
        o.popup = zu.CreateWidget( zu.PopupClass, { content = o.popupContent } )
    end

    return o
end

function SquiggleDecorationClass:reuse()
end

function SquiggleDecorationClass:release()
    local frames = self.frames
    for _,v in pairs( frames ) do
        zu.ReleaseFrame( "squiggles", v )
    end

    if self.popup then
        zu.ReleaseWidget( self.popup )
    end

    if self.popupContent then
        zu.ReleaseWidget( self.popupContent )
    end

    wipe( frames )
end

local function calcXRange( self, lineWidget )
    local charWidth = lineWidget.view.charWidth
    local xstart = 0
    if lineWidget.lineNum == self.firstLine then
        xstart = ( self.firstCol - 1 ) * charWidth
    else
        xstart = 0
    end

    local xend
    if lineWidget.lineNum == self.lastLine then
        xend = self.lastCol * charWidth
    else
        xend = lineWidget.lineLength * charWidth
    end

    return xstart, xend
end

function SquiggleDecorationClass:attach( lineWidget )
    local frame = self.frames[lineWidget] or zu.CreateFrame( "squiggles", "Frame", lineWidget.frame )
    self.frames[lineWidget] = frame

    if not frame.inited then
        frame.tex = frame:CreateTexture( nil, "BACKGROUND" )
        frame.tex:SetAllPoints( frame )
        frame.inited = true
    end

    frame.tex:SetColorTexture( self.r, self.g, self.b, self.a )
    self:update( lineWidget )
    frame:Show()
end

function SquiggleDecorationClass:update( lineWidget )
    local frame = self.frames[lineWidget]
    if not frame then
        return
    end

    local xstart, xend = calcXRange( self, lineWidget )

    frame:SetPoint( "BOTTOMLEFT", lineWidget.frame, "BOTTOMLEFT", xstart, 0 )
    frame:SetPoint( "BOTTOMRIGHT", lineWidget.frame, "BOTTOMLEFT", xend, 0 )
    frame:SetHeight( zu.Scale( 2 ) )
end

function SquiggleDecorationClass:detach( lineWidget )
    zu.ReleaseFrame( "squiggles", self.frames[lineWidget] )
    self.frames[lineWidget] = nil
end

function SquiggleDecorationClass:hitTest( lineWidget, x )
    local xstart, xend = calcXRange( self, lineWidget )
    return x >= xstart and x <= xend
end

function SquiggleDecorationClass:hovering( lw )
    if not self.popup then
        return
    end

    if lw ~= nil then
        local x, y = GetCursorPosition()
        y = 768 - y

        local uiScale = UIParent:GetEffectiveScale()
        x = x / uiScale
        y = y / uiScale

        local xoffset = 20
        local yoffset = -10

        self.popup:show( zu.Scale( x + xoffset ), zu.Scale( y + yoffset ), nil, "BOTTOMLEFT", nil, false )
    else
        self.popup:hide()
    end
end
