--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- A widget that wraps a content widget along with a header,
-- and can collapse its content.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local CollapsibleClass = {}
zu.CollapsibleClass = CollapsibleClass
setmetatable( CollapsibleClass, { __index = zu.CompositeClass } )

function CollapsibleClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.Collapsible( tpl )
    return CollapsibleClass:new( tpl )
end

local cbTpl =
{
    type = "collapsible",
    innerMargin = 4
}

local hiderTpl = {}

function CollapsibleClass:create( parentFrame )
    if not self.content then
        self.settings = {}
        if self.name then
            self.settings = self:getWindow().settings[self.name] or {}
            self:getWindow().settings[self.name] = self.settings
        end

        if self.settings.collapsed ~= nil then
            self.collapsed = self.settings.collapsed
        end

        self.checkBox = zu.CheckBox( cbTpl )
        self.checkBox.label = self.title

        self.checkBox.onClick = function( checked )
            if checked then
                self.collapsed = false
                self.hider:showContent()
            else
                self.hider:hideContent()
                self.collapsed = true
            end

            self.settings.collapsed = self.collapsed
        end

        self.hider = zu.Hider( hiderTpl )
        self.hider.content = self[1]

        self.hider.contentHidden = self.collapsed
        self.checkBox.checked = not self.collapsed

        self.content = zu.VerticalLayout
        {
            self.checkBox,
            self.hider
        }
    end

    self.content.parent = self
    self.content:create( parentFrame )
end
