--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- A widget that can hide its contained widget.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local HiderClass = {}
zu.HiderClass = HiderClass
setmetatable( HiderClass, { __index = zu.CompositeClass } )

HiderClass.contentHidden = false

function HiderClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.Hider( tpl )
    return HiderClass:new( tpl )
end

function HiderClass:create( parentFrame )
    self.content.parent = self
    self.content:create( parentFrame )
end

function HiderClass:show()
    if not self.contentHidden then
        zu.CompositeClass.show( self )
    end
end

function HiderClass:calcMinMax()
    if self.contentHidden then
        self.minWidth = 0
        self.maxWidth = 0
        self.minHeight = 0
        self.maxHeight = 0
        self.preferredWidth = 0
        self.preferredHeight = 0
        self.levelRange = 0
        return
    end

    zu.CompositeClass.calcMinMax( self )
end

function HiderClass:heightForWidth( width )
    if self.contentHidden then
        self.hfw = nil
        return nil
    end

    return zu.CompositeClass.heightForWidth( self, width )
end

function HiderClass:setLevel( level )
    if self.contentHidden then
        return
    end

    zu.CompositeClass.setLevel( self, level )
end

function HiderClass:layout( ... )
    if self.contentHidden then
        zu.Widget.layout( self, ... )
        return
    end

    zu.CompositeClass.layout( self, ... )
end

function HiderClass:hideContent()
    if self.contentHidden then
        return
    end

    self.content:hide()
    self.contentHidden = true
    self:relayout()
end

function HiderClass:showContent()
    if not self.contentHidden then
        return
    end

    self.content:show()
    self.contentHidden = false
    self:relayout()
end
