--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- A scrollable list of elements of fixed height.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local ListClass = {}
zu.ListClass = ListClass
setmetatable( ListClass, { __index = zu.CompositeClass } )

function ListClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.List( s )
    return ListClass:new( s )
end

local function updateSlider( self )
    local height = self.view.height
    self.slider:setTotalAndVisible( self.view:getTotalHeight(), height, height )
    self.slider:setValue( self.view.scrollPosition )
end

local spacerTpl =
{
    minW = 0,
    minH = 0,
    maxH = 1
}

function ListClass:create( parentFrame )
    if not self.content then
        self.model:create()

        self.view = zu.ListView
        {
            model = self.model,
            minDisplayedItems = self.minDisplayedItems,
            preferredDisplayedItems = self.preferredDisplayedItems,
            minimizeHeight = self.minimizeHeight
        }
        zu.AddListener( self.view.updateEvent, self, updateSlider )

        self.slider = zu.VerticalSlider
        {
            background = false,
            fade = true,
            autoFadeOut = true,
            alpha = 0.4,
            onValueUpdated = function( v )
                self.view:setScrollPosition( v )
            end
        }

        self.content = zu.OverlayLayout
        {
            zu.HorizontalLayout
            {
                zu.Spacer ( spacerTpl ),
                self.slider
            },
            self.view
        }

        self.content.parent = self
        self.content:create( parentFrame )

        if self.controller then
            self.controller.enableKeyboardNavigation = self.enableKeyboardNavigation
            self.controller:init( self.view, self.model )
            self.view:setController( self.controller )
            zu.AddListener( self.controller.scrollUpdateEvent, self, updateSlider )
        end
    end
end

function ListClass:layout( x, y, width, height, level )
    zu.CompositeClass.layout( self, x, y, width, height, level )
    self.slider:setTotalAndVisible( self.view:getTotalHeight(), height, height )
end

function ListClass:show()
    zu.CompositeClass.show( self )

    local slidersShown = false
    self.sliderFader = C_Timer.NewTicker( 0.1,
        function()
            if self.frozen then
                return
            end

            if self.view.frame:IsMouseOver() then
                if not slidersShown then
                    self.slider:fadeIn()
                    slidersShown = true
                end
            else
                if slidersShown then
                    self.slider:fadeOut()
                    slidersShown = false
                end
            end
        end )
end

function ListClass:hide()
    if not self.sliderFader then
        return
    end

    self.sliderFader:Cancel()
    self.sliderFader = nil
    zu.CompositeClass.hide( self )
end

function ListClass:release()
    if self.controller then
        self.controller:release()
    end
    self.model:release()

    if self.sliderFader then
        self.sliderFader:Cancel()
        self.sliderFader = nil
    end
    zu.CompositeClass.release( self )
end

function ListClass:freeze()
    zu.CompositeClass.freeze( self )
    if self.controller then
        self.controller:freeze()
    end
end

function ListClass:unfreeze()
    zu.CompositeClass.unfreeze( self )
    if self.controller then
        self.controller:unfreeze()
    end
end
