--
-- Smack, a structured macro language for WoW
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- Base class for selectable list items.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local SelectableItemClass = {}
zu.SelectableItemClass = SelectableItemClass
setmetatable( SelectableItemClass, { __index = zu.CompositeClass } )

function SelectableItemClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function SelectableItemClass:create( parentFrame )
    if not self.decorator then
        self.decorator = self:getSkin().SelItemDecorator:new( self )
    end

    if not self.selFrame then
        self.selFrame = parentFrame:CreateTexture( nil, "BACKGROUND" )
        self.selFrame:Hide()

        self.decorator:update()
    end
end

function SelectableItemClass:release()
    if self.controller then
        zu.RemoveListener( self.controller.selectionChangedEvent, self )
        zu.RemoveListener( self.controller.highlightChangedEvent, self )
    end

    self.selFrame:Hide()
    self.highlighted = false
    self.selected = false
    zu.CompositeClass.release( self )
end

local function selectionUpdate( self )
    local index = self:getIndex()

    if index > 0 and self.controller.selectedItemIndex == index then
        self.selected = true
    else
        self.selected = false
    end

    self.decorator:update()
    self.selFrame:SetShown( self.selected or self.highlighted )
end

function SelectableItemClass:highlight( h )
    self.highlighted = h
    self.decorator:update()
    self.selFrame:SetShown( self.selected or self.highlighted )
end

local function highlightUpdate( self )
    local index = self:getIndex()

    if index > 0 and self.controller.highlightedItemIndex == index then
        if not self.highlighted then
            self:highlight( true )
        end
    else
        if self.highlighted then
            self:highlight( false )
        end
    end
end

function SelectableItemClass:setController( controller )
    if self.controller then
        zu.RemoveListener( self.controller.selectionChangedEvent, self )
        zu.RemoveListener( self.controller.highlightChangedEvent, self )
    end

    self.controller = controller

    if controller then
        zu.AddListener( controller.selectionChangedEvent, self, selectionUpdate )
        zu.AddListener( controller.highlightChangedEvent, self, highlightUpdate )
        selectionUpdate( self )
    end
end

function SelectableItemClass:layout( x, y, width, height, level )
    zu.CompositeClass.layout( self, x, y, width, height, level )
    self.selFrame:SetPoint( "TOPLEFT", self.selFrame:GetParent(), "TOPLEFT", x, -y )
    self.selFrame:SetSize( width, height )
end
