--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local PopupClass = {}
zu.PopupClass = PopupClass

PopupClass.apparitionDelay = 0.4

local popupChannels = {}

function PopupClass:new( tpl, o )
    o = o or {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function PopupClass:reuse()
end

local function getInfos( self )
    local infos = popupChannels[self.channel]
    if not infos then
        infos =
        {
            window = zu.Window { strata = "TOOLTIP" }
        }
        popupChannels[self.channel] = infos
    end

    return infos
end

function zu.Popup( tpl )
    return PopupClass:new( tpl )
end

function PopupClass:show( x, y, frame, pointToAttach, pointToAttachTo, reset )
    local infos = getInfos( self )

    if reset == nil then
        reset = true
    end

    self.requested = true

    if infos.currentWindowOwner == self and ( infos.window.shown or ( not reset and infos.timer ) ) then
        infos.window:setPosition( x, y, frame, pointToAttach, pointToAttachTo )
        return
    end

    infos.window:hide()
    if infos.timer then
        infos.timer:Cancel()
    end

    local showFunc = function()
        infos.window:setDecoratorClass( self.skin:getPopupDecorator( self.style ) )
        self.content.parent = infos.window
        infos.window:setContent( self.content )
        infos.window:show()
        infos.window:setPosition( x, y, frame, pointToAttach, pointToAttachTo )
        self.shown = true
    end

    infos.currentWindowOwner = self

    if self.apparitionDelay > 0 then
        infos.timer = C_Timer.NewTimer( self.apparitionDelay, showFunc )
    else
        showFunc()
    end
end

function PopupClass:hide()
    local infos = getInfos( self )
    self.shown = false
    self.requested = false

    if infos.currentWindowOwner == self then
        infos.window:hide()
        if infos.timer then
            infos.timer:Cancel()
            infos.timer = nil
        end
        infos.currentWindowOwner = nil
    end
end

function PopupClass:isMouseOver()
    if self.attachedPopup and self.attachedPopup:isMouseOver() then
        return true
    end

    local infos = getInfos( self )
    return self.shown and infos.window:isMouseOver()
end

function PopupClass:release()
    self:hide()
end

function PopupClass:freeze()
    self:hide()
end

function PopupClass:resetSize()
    local infos = getInfos( self )
    if infos.currentWindowOwner == self then
        infos.window:resetDefaultSize()
    end
end

function PopupClass:getFrame()
    local infos = getInfos( self )
    if infos.currentWindowOwner == self then
        return infos.window.frame
    end
end
