--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local recycleBins = {}

function zu.CreateWidget( class, tpl )
    tpl = tpl or {}
    local recycleBin = recycleBins[class] or {}
    recycleBins[class] = recycleBin

    if #recycleBin == 0 then
        return class:new( tpl )
    end

    local w = recycleBin[#recycleBin]
    recycleBin[#recycleBin] = nil

    -- replace the template metatable in the reused widget with
    -- the new one
    local mt = getmetatable( tpl ) or {}
    mt.__index = class
    setmetatable( tpl, mt )

    mt = getmetatable( w )
    mt.__index = tpl

    w:reuse( tpl )
    return w
end

function zu.ReleaseWidget( widget, ... )
    widget:release( ... )

    local mt = getmetatable( widget )
    mt = mt.__index
    mt = getmetatable( mt )
    local class = mt.__index

    local recycleBin = recycleBins[class] or {}
    recycleBins[class] = recycleBin

    recycleBin[#recycleBin + 1] = widget
end

local framesRecycleBins = {}

function zu.CreateFrame( pool, type, parent, ... )
    local recycleBin = framesRecycleBins[pool] or {}
    framesRecycleBins[pool] = recycleBin

    if #recycleBin == 0 then
        return CreateFrame( type, nil, parent, ... )
    end

    local f = recycleBin[#recycleBin]
    recycleBin[#recycleBin] = nil

    f:SetParent( parent )
    return f
end

function zu.ReleaseFrame( pool, frame )
    frame:Hide()
    frame:ClearAllPoints()
    frame:SetParent( nil )

    local recycleBin = framesRecycleBins[pool] or {}
    framesRecycleBins[pool] = recycleBin

    recycleBin[#recycleBin + 1] = frame
end

local tablesRecycleBins = {}

function zu.CreateTable( pool, ... )
    local recycleBin = tablesRecycleBins[pool] or {}
    tablesRecycleBins[pool] = recycleBin

    if #recycleBin == 0 then
        return { ... }
    end

    local t = recycleBin[#recycleBin]
    recycleBin[#recycleBin] = nil
    return t
end

function zu.ReleaseTable( pool, table )
    local recycleBin = tablesRecycleBins[pool] or {}
    tablesRecycleBins[pool] = recycleBin

    recycleBin[#recycleBin + 1] = table
end
