--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local wrap = coroutine.wrap
local yield = coroutine.yield

zu.RichTextToken =
{
    Text = 1,
    NewLine = 2,
    TagStart = 3,
    TagEnd = 4
}

local function parseRichText( txt )
    local index = 1
    local verbatimTextMode = false

    while index < #txt do
        local newIndex, text, sep

        if verbatimTextMode then
            _, newIndex, text = txt:find( "([^%$]*)%$", index )
            sep = "$"
        else
            _, newIndex, text, sep = txt:find( "([^%$\n ]*)([%$\n ])", index )
        end

        if newIndex then
            if text ~= "" then
                yield { zu.RichTextToken.Text, text }
            end

            if sep == "\n" then
                yield { zu.RichTextToken.NewLine }
            elseif sep == "$" then
                local endIndex = txt:find( "$", newIndex + 1, true )
                if endIndex then
                    local tag = txt:sub( newIndex + 1, endIndex - 1 )

                    local newVerbatimTextMode
                    if tag == "" then
                        newVerbatimTextMode = yield { zu.RichTextToken.TagEnd }
                    else
                        newVerbatimTextMode = yield { zu.RichTextToken.TagStart, tag }
                    end

                    if newVerbatimTextMode ~= nil then
                        verbatimTextMode = newVerbatimTextMode
                    end
                end

                newIndex = endIndex
            end

            index = newIndex + 1
        else
            text = txt:sub( index )
            if text ~= "" then
                yield { zu.RichTextToken.Text, text }
            end
            break
        end
    end
end

function zu.RichTextParser( txt )
    return wrap( function() parseRichText( txt ) end )
end
