--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local Border = {}
zu.DefaultSkin.Border = Border

-- TODO update method to update the scale (called from layout() probably)
function Border:new( type, frame )
    local o = {}
    setmetatable( o, { __index = self } )

    frame = frame or CreateFrame( "frame", nil, frame )
    o.frame = frame

    local thickness = zu.Scale( 1 )
    local border = true
    if type == "checkbox_hover" then
        border = false
    end

    local bg = frame:CreateTexture( nil, "BACKGROUND" )
    bg:SetDrawLayer( "BACKGROUND", -8 )
    bg:SetAllPoints( frame )
    bg:SetParent( frame )
    o.bg = bg

    if border then
        local top = frame:CreateTexture( nil, "BACKGROUND" )
        top:SetDrawLayer( "BACKGROUND" )
        top:SetPoint( "TOPLEFT", frame )
        top:SetPoint( "BOTTOMRIGHT", frame, "TOPRIGHT", 0, -thickness )

        local bottom = frame:CreateTexture( nil, "BACKGROUND" )
        bottom:SetDrawLayer( "BACKGROUND" )
        bottom:SetPoint( "BOTTOMLEFT", frame )
        bottom:SetPoint( "TOPRIGHT", frame, "BOTTOMRIGHT", 0, thickness )

        local left = frame:CreateTexture( nil, "BACKGROUND" )
        left:SetDrawLayer( "BACKGROUND" )
        left:SetPoint( "TOPLEFT", frame )
        left:SetPoint( "BOTTOMRIGHT", frame, "BOTTOMLEFT", thickness, 0 )

        local right = frame:CreateTexture( nil, "BACKGROUND" )
        right:SetDrawLayer( "BACKGROUND" )
        right:SetPoint( "TOPRIGHT", frame )
        right:SetPoint( "BOTTOMLEFT", frame, "BOTTOMRIGHT", -thickness, 0 )

        o.top = top
        o.bottom = bottom
        o.left = left
        o.right = right
    end

    o:update( type )
    return o
end

function Border:update( type )
    local bgColor = { 0.1, 0, 0.25 }
    local borderColor = { 0.4, 0, 1 }

    if type == "button" then
        bgColor = { 0.15, 0, 0.38 }
        borderColor = { 0.3, 0, 0.75 }
    elseif type == "button_hover" then
        bgColor = { 0.3, 0, 0.75 }
        borderColor = { 0.88, 0.8, 1 }
    elseif type == "button_pushed" then
        bgColor = { 0.1, 0, 0.25 }
        borderColor = { 0.4, 0, 1 }
    elseif type == "button_pushed_hover" then
        bgColor = { 0.2, 0, 0.5 }
        borderColor = { 0.88, 0.8, 1 }
    elseif type == "button_disabled" then
        bgColor = { 0.34, 0.3, 0.38 }
        borderColor = { 0.66, 0.6, 0.75 }
    elseif type == "checkbox_hover" then
        bgColor = { 0.3, 0, 0.75 }
    elseif type == "editbox" then
        bgColor = { 0.05, 0, 0.125 }
        borderColor = { 0.2, 0, 0.5 }
    elseif type == "editbox_hover" then
        bgColor = { 0.05, 0, 0.125 }
        borderColor = { 0.4, 0, 1 }
    elseif type == "popup" then
        bgColor = { 0.1, 0, 0.25 }
        borderColor = { 0.4, 0, 1 }
    end

    self.bg:SetColorTexture( unpack( bgColor ) )

    if self.top then
        self.top:SetColorTexture( unpack( borderColor ) )
        self.bottom:SetColorTexture( unpack( borderColor ) )
        self.left:SetColorTexture( unpack( borderColor ) )
        self.right:SetColorTexture( unpack( borderColor ) )
    end
end

function Border:show()
    self.frame:Show()
end

function Border:hide()
    self.frame:Hide()
end

local windowEdgeColor = { 0.4, 0, 1 }
local windowEdgeColorHighLight = { 0.58, 0.3, 1 }

function Border:setEdgeHighlight( edge, highlighted )

    local color = highlighted and windowEdgeColorHighLight or windowEdgeColor
    local thickness = highlighted and zu.Scale( 2 ) or zu.Scale( 1 )

    local edgeFrame = self[edge]

    if not edgeFrame then
        return
    end

    edgeFrame:SetColorTexture( unpack( color ) )

    if edge == "top" then
        edgeFrame:SetPoint( "BOTTOMRIGHT", edgeFrame:GetParent(), "TOPRIGHT", 0, -thickness )
    elseif edge == "bottom" then
        edgeFrame:SetPoint( "TOPRIGHT", edgeFrame:GetParent(), "BOTTOMRIGHT", 0, thickness )
    elseif edge == "left" then
        edgeFrame:SetPoint( "BOTTOMRIGHT", edgeFrame:GetParent(), "BOTTOMLEFT", thickness, 0 )
    else
        edgeFrame:SetPoint( "BOTTOMLEFT", edgeFrame:GetParent(), "BOTTOMRIGHT", -thickness, 0 )
    end
end

function Border:layout()
    local thickness = zu.Scale( 1 )

    if self.top then
        self.top:SetPoint( "BOTTOMRIGHT", self.top:GetParent(), "TOPRIGHT", 0, -thickness )
    end

    if self.bottom then
        self.bottom:SetPoint( "TOPRIGHT", self.bottom:GetParent(), "BOTTOMRIGHT", 0, thickness )
    end

    if self.left then
        self.left:SetPoint( "BOTTOMRIGHT", self.left:GetParent(), "BOTTOMLEFT", thickness, 0 )
    end

    if self.right then
        self.right:SetPoint( "BOTTOMLEFT", self.right:GetParent(), "BOTTOMRIGHT", -thickness, 0 )
    end
end
