--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local DefaultButtonDecoratorClass = {}
zu.DefaultButtonDecoratorClass = DefaultButtonDecoratorClass

function DefaultButtonDecoratorClass:new( button )
    local o = {}
    o.button = button
    setmetatable( o, { __index = self } )
    return o
end

local function updateState( self, state )
    local enabled = self.frame:IsEnabled()
    local pushed = ( state or self.frame:GetButtonState() ) == "PUSHED"

    if enabled then
        if self.hovering then
            if self.border then
                self.border:update( pushed and "button_pushed_hover" or "button_hover" )
            end

            if self.tex then
                self.tex:SetVertexColor( unpack( zu.DefaultSkin.iconHoverColor ) )
                if pushed then
                    self.tex:AdjustPointsOffset( 2, 2 )
                else
                    self.tex:ClearPointsOffset()
                end
            end
        else
            if self.border then
                self.border:update( pushed and "button_pushed" or "button" )
            end

            if self.tex then
                self.tex:SetVertexColor( unpack( zu.DefaultSkin.iconNormalColor ) )
                self.tex:ClearPointsOffset()
            end
        end
    else
        if self.border then
            self.border:update( "button_disabled" )
        end

        if self.tex then
            self.tex:SetVertexColor( 0.66, 0.6, 0.75 )
            self.tex:ClearPointsOffset()
        end
    end
end

function DefaultButtonDecoratorClass:apply()
    local button = self.button
    self.frame = button.frame
    self.label = button.label

    self.frame:SetNormalFontObject( zu.DefaultSkin.fontNormal )

    self.innerMargin = 10

    if button.type == "action" then
        self.horPadding = 32
    elseif button.type == "keybind" then
        self.innerMargin = 8
    elseif button.type == "delete" or button.type == "trash" then
        self.innerMargin = 0
        self.fixedWidth = true
        self.unframed = true

        local tex, width, height = zu.DefaultSkin:createIconTexture( self.frame, button.type )
        self.tex = tex;
        tex:SetVertexColor( unpack( zu.DefaultSkin.iconNormalColor ) )

        self.wantedWidth = width
        self.wantedHeight = height
    elseif button.type == "closewindow" then
        self.fixedWidth = true
        self.unframed = true
    end

    if self.unframed then
        self.frame:SetScript( "OnMouseDown", function()
            self.button:getWindow():freeze()
        end )

        self.frame:SetScript( "OnMouseUp", function()
            self.button:getWindow():unfreeze()
        end )
    end

    if not self.unframed and not self.border then
        self.border = zu.DefaultSkin.Border:new( "button", self.frame )
    end

    if not self.scriptsAttached then
        self.scriptsAttached = true

        self.frame:SetScript( "OnEnter", function()
            self.hovering = true
            updateState( self )
        end )

        self.frame:SetScript( "OnLeave", function()
            self.hovering = false
            updateState( self )
        end )

        self.frame:SetScript( "OnMouseDown", function()
            self.button:getWindow():freeze()

            -- We ought to get this from button getstate, but it
            -- apparently updates after we receive this so we have
            -- to assume it's pushed here. It's super shitty but
            -- blizzard code does the same.
            -- When in rome and all that
            updateState( self, "PUSHED" )
        end )

        self.frame:SetScript( "OnMouseUp", function()
            updateState( self, "NORMAL" )
            self.button:getWindow():unfreeze()
        end )
    end

    updateState( self, "NORMAL" )
    self.frame:SetText( self.label )
end

function DefaultButtonDecoratorClass:calcMinMax()
    local button = self.button

    if self.frame:GetFontString() then
        self.frame:GetFontString():SetTextScale( zu.ContentScaleFactor() )
    end

    button.minWidth = ( self.wantedWidth and zu.Scale( self.wantedWidth ) or self.frame:GetTextWidth() ) + zu.Scale( self.innerMargin )
    button.minHeight = ( self.wantedHeight and zu.Scale( self.wantedHeight ) or self.frame:GetTextHeight() ) + zu.Scale( self.innerMargin )
    button.maxHeight = button.minHeight

    button.preferredHeight = button.minHeight

    if self.fixedWidth then
        button.preferredWidth = button.minWidth
        button.maxWidth = button.minWidth
    elseif self.button.type == "keybind" then
        -- For the keybind buttons, try to keep a fixed
        -- size that is enough to fit most keybinds, but
        -- grow larger if needed for longer key combinations
        button.minWidth = max( button.minWidth, zu.Scale( 64 ) )
        button.maxWidth = button.minWidth
        button.preferredWidth = button.minWidth
    else
        button.preferredWidth = button.minWidth + ( self.horPadding and zu.Scale( self.horPadding ) or 0 )
    end

    button.levelRange = self.unframed and 1 or 2
end

function DefaultButtonDecoratorClass:heightForWidth()
end

function DefaultButtonDecoratorClass:setLevel( level )
    if self.border then
        self.border.frame:SetFrameLevel( level )
        level = level + 1
    end
    self.frame:SetFrameLevel( level )
end

function DefaultButtonDecoratorClass:layout( x, y, width, height )
    zu.Widget.layout( self, x, y, width, height )
    self.frame:SetSize( width, height )
    self.frame:SetPoint( "TOPLEFT", self.frame:GetParent(), "TOPLEFT", x, -y )

    if self.border then
        self.border:layout( x, y, width, height )
    end
end

function DefaultButtonDecoratorClass:setLabel( label )
    self.label = label
    self.frame:SetText( self.label )
end

function DefaultButtonDecoratorClass:update()
    updateState( self )
end
