--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local DefaultCheckBoxDecoratorClass = {}
zu.DefaultCheckBoxDecoratorClass = DefaultCheckBoxDecoratorClass
setmetatable( DefaultCheckBoxDecoratorClass, { __index = zu.CompositeClass } )

DefaultCheckBoxDecoratorClass.skin = zu.DefaultSkin

function DefaultCheckBoxDecoratorClass:new( checkBox )
    local o = {}
    o.checkBox = checkBox
    setmetatable( o, { __index = self } )
    return o
end

local function updateState( self )
    if self.hovering then
        self.border:show()
    else
        self.border:hide()
    end

    if self.frame:GetChecked() then
        self.content[1].rotateRight90 = self.checkedRotate
        self.content[1]:setIcon( self.checkedIcon )
    else
        self.content[1].rotateRight90 = self.uncheckedRotate
        self.content[1]:setIcon( self.uncheckedIcon )
    end
end
local horBarTpl =
{
    0.2, 0, 0.5, 1,
    minH = 1,
    maxH = 1,
    minW = 1,
    preferredW = 1
}

local chkBoxColor = { 0.53, 0.2, 1 }

function DefaultCheckBoxDecoratorClass:apply()
    local checkBox = self.checkBox
    self.frame = checkBox.frame
    self.parent = self.checkBox

    if checkBox.type == "collapsible" then
        self.checkedIcon = "collapsible"
        self.uncheckedIcon = "collapsible"
        self.checkedRotate = true
    else
        self.checkedIcon = "checkbox-checked"
        self.uncheckedIcon = "checkbox-unchecked"
    end

    if not self.content then
        local icon = zu.Icon { icon = self.uncheckedIcon, color = chkBoxColor }
        self.label = zu.Label { label = checkBox.label }

        if checkBox.type == "collapsible" then
            icon.width = 16
            self.content = zu.HorizontalLayout
            {
                icon,
                zu.Filler( horBarTpl ),
                zu.Minimize { content = self.label },
                zu.Filler( horBarTpl ),
                zu.Spacer { matchSizeOf = icon }
            }
            self.label.justify = "CENTER"
        else
            self.content = zu.HorizontalLayout { spacing = 4, icon, self.label }
        end

        self.content.parent = self
        self.parent = checkBox
        self.content:create( self.frame )
        self.content:show()
    end

    if not self.border then
        self.border = zu.DefaultSkin.Border:new( "checkbox_hover" )
        self.border.frame:SetParent( self.frame )
        self.border.frame:SetAllPoints()
        self.border:hide()
    end

    self.frame:SetScript( "OnEnter", function()
        self.hovering = true
        updateState( self )
    end )

    self.frame:SetScript( "OnLeave", function()
        self.hovering = false
        updateState( self )
    end )

    updateState( self )
end

function DefaultCheckBoxDecoratorClass:calcMinMax()
    zu.CompositeClass.calcMinMax( self )

    if self.checkBox.type == "collapsible" then
        self.minHeight = self.minHeight + zu.Scale( 5 )
        self.maxHeight = self.maxHeight + zu.Scale( 5 )
    end

    self.checkBox.minWidth = self.minWidth
    self.checkBox.maxWidth = self.maxWidth
    self.checkBox.preferredWidth = self.preferredWidth
    self.checkBox.minHeight = self.minHeight
    self.checkBox.maxHeight = self.maxHeight
    self.checkBox.preferredHeight = self.preferredHeight
    self.checkBox.levelRange = 2
end

function DefaultCheckBoxDecoratorClass:heightForWidth( width )
    zu.CompositeClass.heightForWidth( self, width )
end

function DefaultCheckBoxDecoratorClass:setLevel( level )
    self.border.frame:SetFrameLevel( level )
    self.frame:SetFrameLevel( level + 1 )
end

function DefaultCheckBoxDecoratorClass:layout( x, y, width, height )
    if self.checkBox.type == "collapsible" then
        y = y + zu.Scale( 3 )
    end
    zu.CompositeClass.layout( self, x, y, width, height )

    if self.border then
        self.border:layout( x, y, width, height )
    end
end

function DefaultCheckBoxDecoratorClass:update()
    self.label:setLabel( self.checkBox.label )
    updateState( self )
end
