--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local DefaultSkin = zu.DefaultSkin

DefaultSkin.spacing = 4

local syntaxColorPalette =
{
    "|cff8f66cc",   -- Comment
    "|cff79a0ec",   -- ControlStmt
    "|cffec7984",   -- ActionStmt
    "|cff79ece1",   -- Conditional
    "|cffecc579",   -- SpellOrItemName
    "|cff79ec8c",   -- Literal
    "|cffec79d9",   -- Variable
}

local iconIndices =
{
    ["sizingcorner"] = 1,
    ["checkbox-unchecked"] = 2,
    ["checkbox-checked"] = 3,
    ["collapsible"] = 4,
    ["delete"] = 5,
    ["trash"] = 6,
    ["warning"] = 7,
    ["plus"] = 8
}

function DefaultSkin:init()
    if self.inited then
        return
    end

    self.inited = true

    local fontNormal = CreateFont( "SmackFontNormal" )
    fontNormal:SetFont( "Interface/AddOns/Smack_UI/zlui/media/TitilliumWeb-SemiBold.ttf", 15, "" )
    fontNormal:SetTextColor( 0.76, 0.6, 1 )

    local fontEditBoxPrompt = CreateFont( "SmackFontEditBoxPrompt" )
    fontEditBoxPrompt:SetFont( "Interface/AddOns/Smack_UI/zlui/media/TitilliumWeb-Italic.ttf", 13, "" )
    fontEditBoxPrompt:SetTextColor( 0.65, 0.4, 1 )

    self.fontNormal = fontNormal
    self.fontEditBoxPrompt = fontEditBoxPrompt
    self.syntaxColorPalette = syntaxColorPalette
    self.iconsAtlas = zu.iconsAtlas
    self.iconIndices = iconIndices
    self.iconNormalColor = { 0.53, 0.2, 1 }
    self.iconHoverColor = { 0.76, 0.6, 1 }
end

function DefaultSkin:getEditorTextColor()
    return 0.76, 0.6, 1
end

function DefaultSkin:getEditorCursorColor()
    return 0.76, 0.6, 1
end

function DefaultSkin:getEditorSelectionColor()
    return 0.3, 0, 0.75
end

function DefaultSkin:getPopupDecorator( style )
    if style == "tooltip" then
        return zu.DefaultPopupDecoratorClass
    elseif style == "dropdown" then
        return zu.DefaultDropdownDecoratorClass
    elseif style == "doc" then
        return zu.DefaultDocDecoratorClass
    end

    return zu.DefaultWindowDecoratorClass
end

-- Returns MaskTexture, width, height
function DefaultSkin:createIconTexture( frame, iconName, flipX, flipY, rotateRight90, tex )
    local iconInfos = self.iconsAtlas[ self.iconIndices[ iconName ] + 1 ]
    tex = tex or frame:CreateTexture( iconName, "ARTWORK" )
    tex:SetAllPoints()
    tex:SetTexture( "Interface/AddOns/Smack_UI/zlui/media/" .. self.iconsAtlas[1], "CLAMPTOBLACKADDITIVE", "CLAMPTOBLACKADDITIVE" )

    local TopLeftU = iconInfos[3]
    local TopLeftV = iconInfos[4]

    local BottomLeftU = iconInfos[5]
    local BottomLeftV = iconInfos[6]

    local TopRightU = iconInfos[7]
    local TopRightV = iconInfos[8]

    local BottomRightU = iconInfos[9]
    local BottomRightV = iconInfos[10]

    local width = iconInfos[1]
    local height = iconInfos[2]

    if rotateRight90 then
        local tlU, tlV = TopLeftU, TopLeftV
        TopLeftU, TopLeftV = BottomLeftU, BottomLeftV
        BottomLeftU, BottomLeftV = BottomRightU, BottomRightV
        BottomRightU, BottomRightV = TopRightU, TopRightV
        TopRightU, TopRightV = tlU, tlV
        width, height = height, width
    end

    if flipX then
        TopLeftU, TopRightU = TopRightU, TopLeftU
        TopLeftV, TopRightV = TopRightV, TopLeftV
        BottomLeftU, BottomRightU = BottomRightU, BottomLeftU
        BottomLeftV, BottomRightV = BottomRightV, BottomLeftV
    end

    if flipY then
        TopLeftU, BottomLeftU = BottomLeftU, TopLeftU
        TopLeftV, BottomLeftV = BottomLeftV, TopLeftV
        TopRightU, BottomRightU = BottomRightU, TopRightU
        TopRightV, BottomRightV = BottomRightV, TopRightV
    end

    tex:SetTexCoord( TopLeftU, TopLeftV, BottomLeftU, BottomLeftV, TopRightU, TopRightV, BottomRightU, BottomRightV )

    return tex, width, height
end

DefaultSkin.WindowDecorator = zu.DefaultWindowDecoratorClass
DefaultSkin.SashDecorator = zu.DefaultSashDecoratorClass
DefaultSkin.ButtonDecorator = zu.DefaultButtonDecoratorClass
DefaultSkin.CheckBoxDecorator = zu.DefaultCheckBoxDecoratorClass
DefaultSkin.EditBoxDecorator = zu.DefaultEditBoxDecoratorClass
DefaultSkin.LabelDecorator = zu.DefaultLabelDecoratorClass
DefaultSkin.TextDecorator = zu.DefaultTextDecoratorClass
DefaultSkin.SliderDecorator = zu.DefaultSliderDecoratorClass
DefaultSkin.ListViewDecorator = zu.DefaultListViewDecoratorClass
DefaultSkin.SelItemDecorator = zu.DefaultSelItemDecoratorClass
DefaultSkin.EditorViewDecorator = zu.DefaultEditorViewDecoratorClass
