--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local DefaultDropdownDecoratorClass = {}
zu.DefaultDropdownDecoratorClass = DefaultDropdownDecoratorClass
setmetatable( DefaultDropdownDecoratorClass, { __index = zu.CompositeClass } )

function DefaultDropdownDecoratorClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function DefaultDropdownDecoratorClass:release()
    self.client = nil
    self.topLayer[5] = nil
end

function DefaultDropdownDecoratorClass:reuse()
    self.topLayer[5] = self.client
end

function DefaultDropdownDecoratorClass:setClient( client )
    self.client = client
    if not self.content then
        return
    end

    self.topLayer[5] = client
    client.parent = self.content

    self.leftSpacer.matchHeightOf = self.client
    self.sizingHandleRight.matchHeightOf = self.client
    self.topSpacer.matchWidthOf = self.client
    self.sizingHandleBottom.matchWidthOf = self.client
end

local bgFillerTpl =
{
    minW = 1,
    minH = 1
}

local borderSize = 2

local horizontalSpacerTpl =
{
    minW = 1,
    minH = borderSize,
    maxH = borderSize
}

local verticalSpacerTpl =
{
    minW = borderSize,
    minH = 1,
    maxW = borderSize,
}

local horizontalSizeHandleTpl =
{
    minW = 1,
    minH = borderSize,
    maxH = borderSize
}

local verticalSizeHandleTpl =
{
    minW = borderSize,
    minH = 1,
    maxW = borderSize,
    verticalFill = true
}

local cornerSpacerTpl =
{
    minW = borderSize,
    maxW = borderSize,
    minH = borderSize,
    maxH = borderSize
}

function DefaultDropdownDecoratorClass:create( window )
    if not self.content then
        local frame = window.frame

        self.sizingHandleRight = zu.Filler( verticalSizeHandleTpl )
        self.sizingHandleBottom = zu.Filler( horizontalSizeHandleTpl )

        self.leftSpacer = zu.Spacer( verticalSpacerTpl )
        self.topSpacer = zu.Spacer( horizontalSpacerTpl )

        self.leftSpacer.matchHeightOf = self.client
        self.sizingHandleRight.matchHeightOf = self.client
        self.topSpacer.matchWidthOf = self.client
        self.sizingHandleBottom.matchWidthOf = self.client

        self.topLayer = zu.GridLayout
        {
            spacing = 0,
            numColumns = 3,
            zu.Spacer( cornerSpacerTpl ), self.topSpacer, zu.Spacer( cornerSpacerTpl ),
            self.leftSpacer, self.client, self.sizingHandleRight,
            zu.Spacer( cornerSpacerTpl ), self.sizingHandleBottom, zu.Spacer( cornerSpacerTpl ),
        }

        self.content = zu.OverlayLayout
        {
            self.topLayer,
            zu.Filler( bgFillerTpl )
        }

        -- Edge highlighting on hover
        self.sizingHandleRight.onHover =
            function( hover )
                self.border:setEdgeHighlight( "right", hover and self.resizeableWidth )
            end

        self.sizingHandleBottom.onHover =
            function( hover )
                self.border:setEdgeHighlight( "bottom", hover and self.resizeableHeight )
            end

        self.content.parent = window
        self.content:create( frame )

        self.border = zu.DefaultSkin.Border:new( "window", self.content[2].frame )

        self.sizingHandleRight.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "RIGHT" )
            end )

        self.sizingHandleBottom.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "BOTTOM" )
            end )


        self.sizingHandleRight.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )

        self.sizingHandleBottom.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )
    end
end

function DefaultDropdownDecoratorClass:calcMinMax()
    zu.CompositeClass.calcMinMax( self )

    if self.maxWidth and self.maxWidth <= self.minWidth then
        self.resizeableWidth = false
    else
        self.resizeableWidth = true
    end

    if self.maxHeight and self.maxHeight <= self.minHeight then
        self.resizeableHeight = false
    else
        self.resizeableHeight = true
    end
end

function DefaultDropdownDecoratorClass:layout( x, y, width, height )
    zu.CompositeClass.layout( self, x, y, width, height )
    if self.border then
        self.border:layout( x, y, width, height )
    end
end
