--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local DefaultEditBoxDecoratorClass = {}
zu.DefaultEditBoxDecoratorClass = DefaultEditBoxDecoratorClass

function DefaultEditBoxDecoratorClass:new( editbox )
    local o = {}
    o.editbox = editbox
    setmetatable( o, { __index = self } )
    return o
end

local function updateState( self )
    if self.hovering then
        self.border:update( "editbox_hover" )
    else
        self.border:update( "editbox" )
    end
end

-- TODO check when apllied, might need to add a "relayout" method here
-- TODO: apply zu.ContentScale() as the editbox's scale
function DefaultEditBoxDecoratorClass:apply()
    local editbox = self.editbox
    self.frame = editbox.frame
    self.label = editbox.label

    local ebFrame = editbox.editBox
    ebFrame:SetFontObject( zu.DefaultSkin.fontNormal )
    ebFrame:SetTextInsets(0, 0, zu.Scale( 1 ), 0)
    ebFrame:SetHighlightColor( 0.3, 0, 0.75 )
    ebFrame:SetScale( zu.ContentScaleFactor() )

    -- We need a separate fontstring object to get the text height
    -- because this editbox clown show of a widget can't just give
    -- us the height it needs
    if not self.fontStr then
        self.fontStr = self.frame:CreateFontString( nil, "ARTWORK" )
        self.fontStr:SetFontObject( zu.DefaultSkin.fontNormal )
        self.fontStr:SetWordWrap( false )
        self.fontStr:SetText( "W" )
        self.fontStr:Hide()
    end

    if editbox.promptFrame then
        editbox.promptFrame:SetFontObject( zu.DefaultSkin.fontEditBoxPrompt )
    end

    self.innerMargin = 4

    ebFrame:SetPoint( "TOPLEFT", zu.Scale( self.innerMargin ), 0 )
    ebFrame:SetPoint( "BOTTOMRIGHT", zu.Scale( -self.innerMargin ), 0 )

    if not self.border then
        self.border = zu.DefaultSkin.Border:new( "editbox", self.frame )
    end

    ebFrame:SetScript( "OnEnter", function()
        self.hovering = true
        updateState( self )
    end )

    ebFrame:SetScript( "OnLeave", function()
        self.hovering = false
        updateState( self )
    end )
end

function DefaultEditBoxDecoratorClass:calcMinMax()
    local editbox = self.editbox
    self:apply()

    self.fontStr:SetTextScale( zu.ContentScaleFactor() )

    editbox.minWidth = zu.Scale( 32 + self.innerMargin * 2 )
    editbox.preferredWidth = zu.Scale( 48 + self.innerMargin * 2 )

    editbox.minHeight = self.fontStr:GetStringHeight() + zu.Scale( self.innerMargin * 2 )
    editbox.maxHeight = editbox.minHeight
    editbox.preferredHeight = editbox.minHeight
    editbox.levelRange = 1
end

function DefaultEditBoxDecoratorClass:heightForWidth()
end

function DefaultEditBoxDecoratorClass:setLevel( level )
    self.frame:SetFrameLevel( level )
end

function DefaultEditBoxDecoratorClass:layout( x, y, width, height )
    self.frame:SetSize( width, height )
    self.frame:SetPoint( "TOPLEFT", self.frame:GetParent(), "TOPLEFT", x, -y )
    if self.border then
        self.border:layout( x, y, width, height )
    end
end

function DefaultEditBoxDecoratorClass:update()
    updateState( self )
end
