--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local DefaultPopupDecoratorClass = {}
zu.DefaultPopupDecoratorClass = DefaultPopupDecoratorClass
setmetatable( DefaultPopupDecoratorClass, { __index = zu.CompositeClass } )

function DefaultPopupDecoratorClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function DefaultPopupDecoratorClass:release()
    self.client = nil
    self.innerContainer[5] = nil
end

function DefaultPopupDecoratorClass:reuse()
    self.innerContainer[5] = self.client
end

function DefaultPopupDecoratorClass:setClient( client )
    self.client = client
    if not self.content then
        return
    end

    self.innerContainer[5] = client
    client.parent = self.content
end

local borderSize = 3

local cornerSpacerTpl =
{
    minW = borderSize,
    minH = borderSize,
    maxW = borderSize,
    maxH = borderSize,
    preferredW = 1,
    preferredH = 1
}

local verticalSpacerTpl =
{
    minH = 1,
    maxH = 1,
    minW = borderSize,
    maxW = borderSize
}

local horizontalSpacerTpl =
{
    minW = 1,
    maxW = 1,
    minH = borderSize,
    maxH = borderSize
}

local bgTpl =
{
    minW = 1,
    minH = 1
}

function DefaultPopupDecoratorClass:create( window )
    if not self.content then
        local frame = window.frame

        self.innerContainer = zu.GridLayout
        {
            numColumns = 3,
            zu.Spacer( cornerSpacerTpl ), zu.Spacer( horizontalSpacerTpl ), zu.Spacer( cornerSpacerTpl ),
            zu.Spacer( verticalSpacerTpl ), self.client, zu.Spacer( verticalSpacerTpl ),
            zu.Spacer( cornerSpacerTpl )
        }

        self.content = zu.OverlayLayout
        {
            self.innerContainer,
            zu.Filler( bgTpl )
        }

        self.content.parent = window
        self.content:create( frame )

        self.border = zu.DefaultSkin.Border:new( "popup", self.content[2].frame )
    end
end

function DefaultPopupDecoratorClass:layout( x, y, width, height )
    zu.CompositeClass.layout( self, x, y, width, height )
    if self.border then
        self.border:layout( x, y, width, height )
    end
end
