--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local DefaultWindowDecoratorClass = {}
zu.DefaultWindowDecoratorClass = DefaultWindowDecoratorClass
setmetatable( DefaultWindowDecoratorClass, { __index = zu.CompositeClass } )

function DefaultWindowDecoratorClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function DefaultWindowDecoratorClass:release()
    self.client = nil
    self.topLayer[5] = nil
end

function DefaultWindowDecoratorClass:reuse()
    self.topLayer[5] = self.client
end

function DefaultWindowDecoratorClass:setClient( client )
    self.client = client
    if not self.content then
        return
    end

    self.topLayer[5] = client
    client.parent = self.content
end

local bgFillerTpl =
{
    minW = 1,
    minH = 1
}

local borderSize = 7

local horizontalSpacerTpl =
{
    minW = 1,
    minH = borderSize,
    maxH = borderSize
}

local verticalSpacerTpl =
{
    minW = borderSize,
    minH = 1,
    maxW = borderSize,
}

local centerSpacerTpl =
{
    minW = 1,
    minH = 1
}

local cornerSpacerTpl =
{
    minW = borderSize,
    maxW = borderSize,
    minH = borderSize,
    maxH = borderSize
}

local cornerSizeHandleTpl =
{
    icon = "sizingcorner",
    color = { 0, 0, 0, 0 },
    highlightColor = { 0.58, 0.3, 1, 1 }
}

local cornerSizeHandleBRTpl =
{
    icon = "sizingcorner",
    color = { 0.4, 0, 1, 1 },
    highlightColor = { 0.58, 0.3, 1, 1 }
}

function DefaultWindowDecoratorClass:create( window )
    if not self.content then
        local frame = window.frame

        self.titleBar = zu.Label
        {
            label = window.title,
            justify = "CENTER"
        }

        self.aboutButton = zu.Button
        {
            label = "?",
            type = "closewindow",
            onClick = function()
                self.parent:about()
            end
        }

        self.closeButton = zu.Button
        {
            label = "x",
            type = "closewindow",
            onClick = function()
                self.parent:hide()
            end
        }

        self.sizingHandleTL = zu.Icon( cornerSizeHandleTpl )
        self.sizingHandleTR = zu.Icon( cornerSizeHandleTpl )
        self.sizingHandleBL = zu.Icon( cornerSizeHandleTpl )
        self.sizingHandleBR = zu.Icon( cornerSizeHandleBRTpl )

        self.sizingHandleTL.flipX = true
        self.sizingHandleTL.flipY = true
        self.sizingHandleTR.flipY = true
        self.sizingHandleBL.flipX = true

        self.sizingHandleLeft = zu.Filler{}
        self.sizingHandleRight = zu.Filler{}
        self.sizingHandleTop = zu.Filler{}
        self.sizingHandleBottom = zu.Filler{}

        self.sizingHandleLeft.matchWidthOf = self.sizingHandleTL
        self.sizingHandleRight.matchWidthOf = self.sizingHandleTR
        self.sizingHandleTop.matchHeightOf = self.sizingHandleTL
        self.sizingHandleBottom.matchHeightOf = self.sizingHandleBL

        self.topLayer = zu.GridLayout
        {
            spacing = 0,
            numColumns = 3,
            zu.Spacer( cornerSpacerTpl ), zu.HorizontalLayout{ self.titleBar, self.aboutButton, self.closeButton }, zu.Spacer( cornerSpacerTpl ),
            zu.Spacer( verticalSpacerTpl ), self.client, zu.Spacer( verticalSpacerTpl ),
            zu.Spacer( cornerSpacerTpl ), zu.Spacer( horizontalSpacerTpl ), zu.Spacer( cornerSpacerTpl ),
        }

        self.sizeHandlesLayer = zu.GridLayout
        {
            spacing = 0,
            numColumns = 3,
            self.sizingHandleTL, self.sizingHandleTop, self.sizingHandleTR,
            self.sizingHandleLeft, zu.Spacer( centerSpacerTpl ), self.sizingHandleRight,
            self.sizingHandleBL, self.sizingHandleBottom, self.sizingHandleBR
        }

        self.content = zu.OverlayLayout
        {
            self.topLayer,
            self.sizeHandlesLayer,
            zu.Filler( bgFillerTpl )
        }

        -- Edge highlighting on hover
        self.sizingHandleLeft.onHover =
            function( hover )
                self.border:setEdgeHighlight( "left", hover )
            end

        self.sizingHandleRight.onHover =
            function( hover )
                self.border:setEdgeHighlight( "right", hover )
            end

        self.sizingHandleTop.onHover =
            function( hover )
                self.border:setEdgeHighlight( "top", hover )
            end

        self.sizingHandleBottom.onHover =
            function( hover )
                self.border:setEdgeHighlight( "bottom", hover )
            end

        self.content.parent = window
        self.content:create( frame )

        self.border = zu.DefaultSkin.Border:new( "window", self.content[3].frame )

        self.titleBar.frame:SetScript( "OnMouseDown",
            function()
                self.parent.frame:StartMoving()
            end )

        self.titleBar.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )


        self.sizingHandleTL.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "TOPLEFT" )
            end )

        self.sizingHandleTR.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "TOPRIGHT" )
            end )

        self.sizingHandleBL.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "BOTTOMLEFT" )
            end )

        self.sizingHandleBR.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "BOTTOMRIGHT" )
            end )


        self.sizingHandleTL.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )

        self.sizingHandleTR.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )

        self.sizingHandleBL.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )

        self.sizingHandleBR.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )


        self.sizingHandleLeft.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "LEFT" )
            end )

        self.sizingHandleRight.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "RIGHT" )
            end )

        self.sizingHandleTop.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "TOP" )
            end )

        self.sizingHandleBottom.frame:SetScript( "OnMouseDown",
            function()
                self.parent:startSizing( "BOTTOM" )
            end )


        self.sizingHandleLeft.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )

        self.sizingHandleRight.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )

        self.sizingHandleTop.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )

        self.sizingHandleBottom.frame:SetScript( "OnMouseUp",
            function()
                self.parent:stopMovingOrSizing()
            end )
    end
end

function DefaultWindowDecoratorClass:layout( x, y, width, height )
    zu.CompositeClass.layout( self, x, y, width, height )
    if self.border then
        self.border:layout( x, y, width, height )
    end
end
