--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- A widget to edit a set of tags.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local TagEditorClass = {}
zu.TagEditorClass = TagEditorClass
setmetatable( TagEditorClass, { __index = zu.CompositeClass } )

function TagEditorClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.TagEditor( tpl )
    return TagEditorClass:new( tpl )
end

function TagEditorClass:create( parentFrame )
    if not self.content then
        self.editBox = zu.EditBox
        {
            prompt = "Enter tags separated by commas",
            onTextEditionFinished = function( text )
                if not self.model then
                    return
                end

                local tagList = { strsplit( ",", text ) }

                for i = 1, #tagList, 1 do
                    local tag = strtrim( tagList[i] )
                    if tag ~= "" then
                        self.model:add( tag )
                    end
                end

                self.editBox:setText( "" )
            end
        }

        self.view = zu.TagView
        {
            tagWidgetClass = zu.TagEditorItemClass,
            colorizer = self.colorizer
        }

        if self.model then
            self.view:setModel( self.model )
        end

        self.content = zu.VerticalLayout
        {
            self.editBox,
            self.view
        }

        self.content.parent = self
        self.content:create( parentFrame )
    end
end

function TagEditorClass:setModel( tagSet )
    if self.model == tagSet then
        return
    end

    zu.Tags:new( tagSet )

    self.model = tagSet
    self.editBox:setText( "" )

    if self.view then
        self.view:setModel( tagSet )
    end
end
