--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- An adapter that exposes a tag set as a model that
-- can be used in a listview.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local TagListModel = {}
zu.TagListModel = TagListModel

local function update( self )
    zu.Raise( self.updateEvent )
end

function TagListModel:new( tagSet )
    local o = {}

    o.updateEvent = {}
    o.tags = tagSet

    setmetatable( o, { __index = self } )
    return o
end

function TagListModel:create()
    if self.initDone then
        return
    end

    self.initDone = true

    zu.AddListener( self.tags.tagAddedEvent, self, update )
    zu.AddListener( self.tags.tagRemovedEvent, self, update )
    zu.AddListener( self.tags.tagsUpdatedEvent, self, update )
end

function TagListModel:release()
    zu.RemoveListener( self.tags.tagAddedEvent, self )
    zu.RemoveListener( self.tags.tagRemovedEvent, self )
    zu.RemoveListener( self.tags.tagsUpdatedEvent, self )
end

function TagListModel:count()
    return #self.tags
end

function TagListModel:getElementSize( view )
    local w = zu.CreateWidget( self.elementWidgetClass )
    w.parent = view
    w.owner = self
    w.tag = "W"
    w:create( view.frame )
    w:calcMinMax()

    local height, minWidth, prefWidth, maxWidth, levelRange =
        w.minHeight, w.minWidth, w.preferredWidth, w.maxWidth, w.levelRange
    zu.ReleaseWidget( w )

    return height, minWidth, prefWidth, maxWidth, levelRange
end

function TagListModel:getWidget( index )
    local w = zu.CreateWidget( self.elementWidgetClass )
    w.owner = self
    w.tag = self.tags[index] or "W"
    return w
end

function TagListModel:getElement( index )
    return self.tags[index]
end
