--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- A widget to select tags in a set of tags.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local TagSelectorClass = {}
zu.TagSelectorClass = TagSelectorClass
setmetatable( TagSelectorClass, { __index = zu.CompositeClass } )

function TagSelectorClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    return o
end

function zu.TagSelector( tpl )
    return TagSelectorClass:new( tpl )
end

function TagSelectorClass:create( parentFrame )
    if not self.content then
        self.listModel = zu.TagListModel:new( self.model )
        self.listModel.elementWidgetClass = zu.TagSelectorItemClass
        self.listModel.colorizer = self.colorizer

        self.content = zu.List
        {
            model = self.listModel,
            minimizeHeight = true
        }

        self.content.parent = self
        self.content:create( parentFrame )
    end

    if self.outputModel then
        self.listModel.outputModel = self.outputModel
    end
end

local function tagRemoved( self, _, tag )
    self.outputModel:remove( tag )
end

function TagSelectorClass:setModel( tagSet )
    if self.model then
        zu.RemoveListener( self.model.tagRemovedEvent, self )
    end
    self.model = tagSet

    zu.AddListener( tagSet.tagRemovedEvent, self, tagRemoved )
end

function TagSelectorClass:setOutputModel( tagSet )
    self.outputModel = tagSet
    if self.content then
        self.content.outputModel = tagSet
    end
end
