--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- A set of tags.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local remove = table.remove
local sort = table.sort

local Tags = {}
zu.Tags = Tags

function Tags:new( tagArray )
    tagArray = tagArray or {}
    local metaData = getmetatable( tagArray )
    if tagArray.dict then
        return tagArray
    end

    metaData = metaData and metaData.__index or {}

    metaData.dict = {}
    metaData.tagAddedEvent = metaData.tagAddedEvent or {}
    metaData.tagRemovedEvent = metaData.tagRemovedEvent or {}
    metaData.tagsUpdatedEvent = metaData.tagsUpdatedEvent or {}

    setmetatable( tagArray, { __index = metaData } )
    setmetatable( metaData, { __index = self } )

    for i = 1, #tagArray, 1 do
        local lc = tagArray[i]:lower()
        tagArray.dict[lc] = i
    end

    return tagArray
end

function Tags:add( tag )
    local lc = tag:lower()
    if self.dict[lc] then
        return
    end

    self[#self+1] = tag
    self.dict[lc] = #self

    zu.Raise( self.tagAddedEvent, #self, tag )
end

function Tags:remove( tag )
    local lc = tag:lower()
    local index = self.dict[lc]

    if index ~= nil then
        self.dict[lc] = nil
        remove( self, index )

        for k, i in pairs( self.dict ) do
            if i > index then
                self.dict[k] = i - 1
            end
        end

        zu.Raise( self.tagRemovedEvent, index, tag )
    end
end

function Tags:merge( tags )
    for i = 1, #tags, 1 do
        self:add( tags[i] )
    end
end

function Tags:isSubSetOf( tags )
    for i = 1, #self, 1 do
        local lc = self[i]:lower()
        if tags.dict[lc] == nil then
            return false
        end
    end

    return true
end

function Tags:contains( tag )
    local lc = tag:lower()
    return self.dict[lc] ~= nil
end

function Tags:sort()
    sort( self )

    for i = 1, #self, 1 do
        local lc = self[i]:lower()
        self.dict[lc] = i
    end

    zu.Raise( self.tagsUpdatedEvent )
end
