--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
-- A tag set view. The widget class used to represent a tag
-- is provided externally, so this can be used both by the tag editor
-- and by the tag selector.
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local remove = table.remove

local TagViewClass = {}
zu.TagViewClass = TagViewClass
setmetatable( TagViewClass, { __index = zu.FlowLayoutClass } )

function TagViewClass:new( tpl )
    local o = {}
    setmetatable( tpl, { __index = self } )
    setmetatable( o, { __index = tpl } )
    o.numElements = 0
    return o
end

function zu.TagView( tpl )
    return TagViewClass:new( tpl )
end

function TagViewClass:release()
    zu.FlowLayoutClass.release( self )
    zu.RemoveListener( self.model.tagAddedEvent, self )
    zu.RemoveListener( self.model.tagRemovedEvent, self )
    zu.RemoveListener( self.model.tagsUpdatedEvent, self )

    self.numElements = 0
end

local function addTag( self, _, tag )
    local w = zu.CreateWidget( self.tagWidgetClass )
    w.parent = self

    self[self.numElements] = w
    self.numElements = self.numElements + 1
    self[self.numElements] = self.spacer

    w.owner = self
    w.tag = tag

    w:create( self.parentFrame )

    if self.shown then
        w:show()
    else
        w:hide()
    end
end

local function onTagAdded( self, index, tag )
    if self.spacer then
        addTag( self, index, tag )
        self:relayout()
    end
end

local function onTagRemoved( self, index )
    if self.spacer then
        local w = self[index]

        remove( self, index )
        self.numElements = self.numElements - 1

        zu.ReleaseWidget( w )
        self:relayout()
    end
end

local function createAllWidgets( self )
    for i = 1, #self.model, 1 do
        addTag( self, i, self.model[i] )
    end

    self:relayout()
end

local function onTagsUpdated( self )
    if self.spacer then
        self.numElements = 1
        self[1] = self.spacer
        createAllWidgets( self )
        self:relayout()
    end
end

function TagViewClass:setModel( tagSet )
    if self.model == tagSet then
        return
    end

    if self.model then
        zu.RemoveListener( self.model.tagAddedEvent, self )
        zu.RemoveListener( self.model.tagRemovedEvent, self )
        zu.RemoveListener( self.model.tagsUpdatedEvent, self )
    end

    self.model = tagSet

    for i = 1, self.numElements - 1, 1 do
        zu.ReleaseWidget( self[i] )
    end

    if self.spacer then
        self.numElements = 1
        self[1] = self.spacer
    end

    if tagSet == nil then
        self:relayout()
        return
    end

    zu.AddListener( tagSet.tagAddedEvent, self, onTagAdded )
    zu.AddListener( tagSet.tagRemovedEvent, self, onTagRemoved )
    zu.AddListener( tagSet.tagsUpdatedEvent, self, onTagsUpdated )

    if self.spacer then
        createAllWidgets( self )
    end
end

local spacerTpl = { minW = 0, minH = 0, maxH = 0 }

function TagViewClass:create( parentFrame )
    if not self.spacer then
        self.spacer = zu.Spacer( spacerTpl )
        self.spacer.parent = self
        self.parentFrame = parentFrame

        self[1] = self.spacer
        self.numElements = 1

        zu.FlowLayoutClass.create( self, parentFrame )

        if self.model then
            createAllWidgets( self )
        end
    end
end
