--
-- Zlodo's UI framework
-- Author: Zlodo (Taliriah-Silvermoon EU)
--
local _, addon = ...
addon.zu = addon.zu or {}
local zu = addon.zu

local Widget = {}
zu.Widget = Widget

Widget.levelRange = 0

-- Pre-declare the default skin,
-- but it will be filled in later
zu.DefaultSkin = {}

function zu.ScaleFactor()
    return PixelUtil.GetPixelToUIUnitFactor()
end

-- The scale factor to apply to the content. Should always be rounded
-- to a pixel size after scaling.
local function ContentScaleFactor()
    local _, screenHeight = GetPhysicalScreenSize()
    local contentScaleFactor = screenHeight / 768

    contentScaleFactor = contentScaleFactor * UIParent:GetEffectiveScale()

    return contentScaleFactor
end

zu.ContentScaleFactor = ContentScaleFactor

-- Common utility function to scale the size of a content element
-- (text, separator, etc.)
function zu.Scale( value )
    value = value * ContentScaleFactor()
    value = math.ceil( value )
    return value
end

function Widget:release()
    self:hide()
    self.width = nil
end

function Widget:reuse()
end

function Widget:show()
    self.shown = true

    if not self.frame then
        self:create()
    end

    self.frame:Show()
end

function Widget:hide()
    self.shown = false

    if self.frame then
        self.frame:Hide()
    end
end

function Widget:freeze()
    self.frozen = true
end

function Widget:unfreeze()
    self.frozen = false
end

function Widget:heightForWidth()
end

function Widget:setLevel( level )
    self.level = level
end

function Widget:layout( x, y, width, height )
    self.x = x
    self.y = y
    self.width = width
    self.height = height
end

function Widget:getPreferredWidth()
    return self.preferredWidth
end

function Widget:getPreferredHeight()
    return self.preferredHeight
end

function Widget:getMinWidth()
    return self.minWidth
end

function Widget:getMinHeight()
    return self.minHeight
end

function Widget:getMaxWidth()
    return self.maxWidth
end

function Widget:getMaxHeight()
    return self.maxHeight
end

function Widget:setWidth( width )
    self.width = width
end

function Widget:setHeight( height )
    self.height = height
end

-- Retrieve the containing window
function Widget:getWindow()
    local w = self.parent

    while w.parent ~= nil do
        w = w.parent
    end

    return w
end

-- If the min/max of the widget change or may potentially change
-- (for instance setting a new label in a button), call this.
--
-- This will recompute the widget's minmax and width for height.
-- If they are unchanged, it will call layout to relayout the widget.
--
-- Otherwise, it will escalate the relayout request by calling the parent
-- widget's relayout method, causing neighboring widgets to be resized
-- to give us the necessary additional space, and so on.
--
-- If necessary, it will be escalated all the way to the containing window,
-- which will resize itself accordingly.
function Widget:relayout()
    if self.width == nil or self.height == nil then
        -- If we were never laid out yet in the first place,
        -- ignore the relayout request.
        return
    end

    local oldMinWidth = self.minWidth
    local oldMaxWidth = self.maxWidth

    local oldMinHeight = self.minHeight
    local oldMaxHeight = self.maxHeight

    local oldHFW = self.hfw

    local oldLevelRange = self.levelRange

    self:calcMinMax()

    if  oldMinWidth ~= self.minWidth or
        oldMaxWidth ~= self.maxWidth or
        oldMinHeight ~= self.minHeight or
        oldMaxHeight ~= self.maxHeight or
        oldLevelRange ~= self.levelRange then
        if self.parent then
            self.parent:relayout()
        end
        return
    end

    self:heightForWidth( self.width )

    if oldHFW ~= self.hfw then
        if self.parent then
            self.parent:relayout()
        end
        return
    end

    if self.level then
        self:setLevel( self.level )
    end
    self:layout( self.x, self.y, self.width, self.height )
end

function Widget:getSkin()
    if not self.skin then
        self.skin = self.parent:getSkin()
    end

    return self.skin
end
